@@ -24,6 +24,7 @@
 
 class QLocalServer;
 class QLocalSocket;
+class QString;
 
 class BrowserHost : public QObject
 {
@@ -36,16 +37,20 @@ public:
     void start();
     void stop();
 
-    void sendClientMessage(const QJsonObject& json);
+    void broadcastClientMessage(const QJsonObject& json);
+    void sendClientMessage(QLocalSocket* socket, const QJsonObject& json);
 
 signals:
-    void clientMessageReceived(const QJsonObject& json);
+    void clientMessageReceived(QLocalSocket* socket, const QJsonObject& json);
 
 private slots:
     void proxyConnected();
     void readProxyMessage();
     void proxyDisconnected();
 
+private:
+    void sendClientData(QLocalSocket* socket, const QString& data);
+
 private:
     QPointer<QLocalServer> m_localServer;
     QList<QLocalSocket*> m_socketList;
