@@ -1,6 +1,6 @@
 /*
  *  Copyright (C) 2010 Felix Geyer <debfx@fobos.de>
- *  Copyright (C) 2017 KeePassXC Team <team@keepassxc.org>
+ *  Copyright (C) 2021 KeePassXC Team <team@keepassxc.org>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,20 +19,21 @@
 #include "Group.h"
 #include "config-keepassx.h"
 
-#include "core/Clock.h"
 #include "core/Config.h"
-#include "core/DatabaseIcons.h"
-#include "core/Global.h"
-#include "core/Metadata.h"
-#include "core/Tools.h"
 
 #ifdef WITH_XC_KEESHARE
 #include "keeshare/KeeShare.h"
 #endif
 
+#include "core/Global.h"
+#include "core/Metadata.h"
+#include "core/Tools.h"
+
 #include <QtConcurrent>
+#include <QtConcurrentFilter>
 
 const int Group::DefaultIconNumber = 48;
+const int Group::OpenFolderIconNumber = 49;
 const int Group::RecycleBinIconNumber = 43;
 const QString Group::RootAutoTypeSequence = "{USERNAME}{TAB}{PASSWORD}{ENTER}";
 
@@ -46,9 +47,9 @@ Group::Group()
     m_data.searchingEnabled = Inherit;
     m_data.mergeMode = Default;
 
-    connect(m_customData, SIGNAL(customDataModified()), this, SIGNAL(groupModified()));
-    connect(this, SIGNAL(groupModified()), SLOT(updateTimeinfo()));
-    connect(this, SIGNAL(groupNonDataChange()), SLOT(updateTimeinfo()));
+    connect(m_customData, &CustomData::modified, this, &Group::modified);
+    connect(this, &Group::modified, this, &Group::updateTimeinfo);
+    connect(this, &Group::groupNonDataChange, this, &Group::updateTimeinfo);
 }
 
 Group::~Group()
@@ -80,7 +81,7 @@ template <class P, class V> inline bool Group::set(P& property, const V& value)
 {
     if (property != value) {
         property = value;
-        emit groupModified();
+        emitModified();
         return true;
     } else {
         return false;
@@ -125,42 +126,9 @@ QString Group::notes() const
     return m_data.notes;
 }
 
-QImage Group::icon() const
+QString Group::tags() const
 {
-    if (m_data.customIcon.isNull()) {
-        return databaseIcons()->icon(m_data.iconNumber).toImage();
-    } else {
-        Q_ASSERT(m_db);
-        if (m_db) {
-            return m_db->metadata()->customIcon(m_data.customIcon);
-        } else {
-            return QImage();
-        }
-    }
-}
-
-QPixmap Group::iconPixmap(IconSize size) const
-{
-    QPixmap icon(size, size);
-    if (m_data.customIcon.isNull()) {
-        icon = databaseIcons()->icon(m_data.iconNumber, size);
-    } else {
-        Q_ASSERT(m_db);
-        if (m_db) {
-            icon = m_db->metadata()->customIconPixmap(m_data.customIcon, size);
-        }
-    }
-
-    if (isExpired()) {
-        icon = databaseIcons()->applyBadge(icon, DatabaseIcons::Badges::Expired);
-    }
-#ifdef WITH_XC_KEESHARE
-    else if (KeeShare::isShared(this)) {
-        icon = KeeShare::indicatorBadge(this, icon);
-    }
-#endif
-
-    return icon;
+    return m_data.tags;
 }
 
 int Group::iconNumber() const
@@ -278,6 +246,35 @@ const CustomData* Group::customData() const
     return m_customData;
 }
 
+Group::TriState Group::resolveCustomDataTriState(const QString& key, bool checkParent) const
+{
+    // If not defined, check our parent up to the root group
+    if (!m_customData->contains(key)) {
+        if (!m_parent || !checkParent) {
+            return Inherit;
+        } else {
+            return m_parent->resolveCustomDataTriState(key);
+        }
+    }
+
+    return m_customData->value(key) == TRUE_STR ? Enable : Disable;
+}
+
+void Group::setCustomDataTriState(const QString& key, const Group::TriState& value)
+{
+    switch (value) {
+    case Enable:
+        m_customData->set(key, TRUE_STR);
+        break;
+    case Disable:
+        m_customData->set(key, FALSE_STR);
+        break;
+    case Inherit:
+        m_customData->remove(key);
+        break;
+    }
+}
+
 bool Group::equals(const Group* other, CompareItemOptions options) const
 {
     if (!other) {
@@ -328,12 +325,17 @@ void Group::setNotes(const QString& notes)
     set(m_data.notes, notes);
 }
 
+void Group::setTags(const QString& tags)
+{
+    set(m_data.tags, tags);
+}
+
 void Group::setIcon(int iconNumber)
 {
     if (iconNumber >= 0 && (m_data.iconNumber != iconNumber || !m_data.customIcon.isNull())) {
         m_data.iconNumber = iconNumber;
         m_data.customIcon = QUuid();
-        emit groupModified();
+        emitModified();
         emit groupDataChanged(this);
     }
 }
@@ -343,7 +345,7 @@ void Group::setIcon(const QUuid& uuid)
     if (!uuid.isNull() && m_data.customIcon != uuid) {
         m_data.customIcon = uuid;
         m_data.iconNumber = 0;
-        emit groupModified();
+        emitModified();
         emit groupDataChanged(this);
     }
 }
@@ -385,7 +387,7 @@ void Group::setExpires(bool value)
 {
     if (m_data.timeInfo.expires() != value) {
         m_data.timeInfo.setExpires(value);
-        emit groupModified();
+        emitModified();
     }
 }
 
@@ -393,7 +395,7 @@ void Group::setExpiryTime(const QDateTime& dateTime)
 {
     if (m_data.timeInfo.expiryTime() != dateTime) {
         m_data.timeInfo.setExpiryTime(dateTime);
-        emit groupModified();
+        emitModified();
     }
 }
 
@@ -412,7 +414,7 @@ const Group* Group::parentGroup() const
     return m_parent;
 }
 
-void Group::setParent(Group* parent, int index)
+void Group::setParent(Group* parent, int index, bool trackPrevious)
 {
     Q_ASSERT(parent);
     Q_ASSERT(index >= -1 && index <= parent->children().size());
@@ -437,12 +439,13 @@ void Group::setParent(Group* parent, int index)
         cleanupParent();
         m_parent = parent;
         if (m_db) {
+            setPreviousParentGroup(nullptr);
             recCreateDelObjects();
 
             // copy custom icon to the new database
             if (!iconUuid().isNull() && parent->m_db && m_db->metadata()->hasCustomIcon(iconUuid())
                 && !parent->m_db->metadata()->hasCustomIcon(iconUuid())) {
-                parent->m_db->metadata()->addCustomIcon(iconUuid(), icon());
+                parent->m_db->metadata()->addCustomIcon(iconUuid(), m_db->metadata()->customIcon(iconUuid()));
             }
         }
         if (m_db != parent->m_db) {
@@ -454,6 +457,9 @@ void Group::setParent(Group* parent, int index)
         parent->m_children.insert(index, this);
     } else {
         emit aboutToMove(this, parent, index);
+        if (trackPrevious && m_parent != parent) {
+            setPreviousParentGroup(m_parent);
+        }
         m_parent->m_children.removeAll(this);
         m_parent = parent;
         QObject::setParent(parent);
@@ -465,7 +471,7 @@ void Group::setParent(Group* parent, int index)
         m_data.timeInfo.setLocationChanged(Clock::currentDateTimeUtc());
     }
 
-    emit groupModified();
+    emitModified();
 
     if (!moveWithinDatabase) {
         emit groupAdded();
@@ -594,7 +600,7 @@ Entry* Group::findEntryByUuid(const QUuid& uuid, bool recursive) const
     return nullptr;
 }
 
-Entry* Group::findEntryByPath(const QString& entryPath)
+Entry* Group::findEntryByPath(const QString& entryPath) const
 {
     if (entryPath.isEmpty()) {
         return nullptr;
@@ -656,7 +662,7 @@ Entry* Group::findEntryBySearchTerm(const QString& term, EntryReferenceType refe
     return nullptr;
 }
 
-Entry* Group::findEntryByPathRecursive(const QString& entryPath, const QString& basePath)
+Entry* Group::findEntryByPathRecursive(const QString& entryPath, const QString& basePath) const
 {
     // Return the first entry that matches the full path OR if there is no leading
     // slash, return the first entry title that matches
@@ -852,6 +858,21 @@ Group* Group::findGroupByUuid(const QUuid& uuid)
     return nullptr;
 }
 
+const Group* Group::findGroupByUuid(const QUuid& uuid) const
+{
+    if (uuid.isNull()) {
+        return nullptr;
+    }
+
+    for (const Group* group : groupsRecursive(true)) {
+        if (group->uuid() == uuid) {
+            return group;
+        }
+    }
+
+    return nullptr;
+}
+
 Group* Group::findChildByName(const QString& name)
 {
     for (Group* group : asConst(m_children)) {
@@ -907,6 +928,10 @@ Group* Group::clone(Entry::CloneFlags entryFlags, Group::CloneFlags groupFlags)
         clonedGroup->m_data.timeInfo.setLocationChanged(now);
     }
 
+    if (groupFlags & Group::CloneRenameTitle) {
+        clonedGroup->setName(tr("%1 - Clone").arg(name()));
+    }
+
     return clonedGroup;
 }
 
@@ -927,12 +952,12 @@ void Group::addEntry(Entry* entry)
     emit entryAboutToAdd(entry);
 
     m_entries << entry;
-    connect(entry, SIGNAL(entryDataChanged(Entry*)), SIGNAL(entryDataChanged(Entry*)));
+    connect(entry, &Entry::entryDataChanged, this, &Group::entryDataChanged);
     if (m_db) {
-        connect(entry, SIGNAL(entryModified()), m_db, SLOT(markAsModified()));
+        connect(entry, &Entry::modified, m_db, &Database::markAsModified);
     }
 
-    emit groupModified();
+    emitModified();
     emit entryAdded(entry);
 }
 
@@ -949,7 +974,7 @@ void Group::removeEntry(Entry* entry)
         entry->disconnect(m_db);
     }
     m_entries.removeAll(entry);
-    emit groupModified();
+    emitModified();
     emit entryRemoved(entry);
 }
 
@@ -990,21 +1015,21 @@ void Group::connectDatabaseSignalsRecursive(Database* db)
             entry->disconnect(m_db);
         }
         if (db) {
-            connect(entry, SIGNAL(entryModified()), db, SLOT(markAsModified()));
+            connect(entry, &Entry::modified, db, &Database::markAsModified);
         }
     }
 
     if (db) {
         // clang-format off
-        connect(this, SIGNAL(groupDataChanged(Group*)), db, SIGNAL(groupDataChanged(Group*)));
-        connect(this, SIGNAL(groupAboutToRemove(Group*)), db, SIGNAL(groupAboutToRemove(Group*)));
-        connect(this, SIGNAL(groupRemoved()), db, SIGNAL(groupRemoved()));
-        connect(this, SIGNAL(groupAboutToAdd(Group*, int)), db, SIGNAL(groupAboutToAdd(Group*,int)));
-        connect(this, SIGNAL(groupAdded()), db, SIGNAL(groupAdded()));
-        connect(this, SIGNAL(aboutToMove(Group*,Group*,int)), db, SIGNAL(groupAboutToMove(Group*,Group*,int)));
-        connect(this, SIGNAL(groupMoved()), db, SIGNAL(groupMoved()));
-        connect(this, SIGNAL(groupModified()), db, SLOT(markAsModified()));
-        connect(this, SIGNAL(groupNonDataChange()), db, SLOT(markNonDataChange()));
+        connect(this, &Group::groupDataChanged, db, &Database::groupDataChanged);
+        connect(this, &Group::groupAboutToRemove, db, &Database::groupAboutToRemove);
+        connect(this, &Group::groupRemoved, db, &Database::groupRemoved);
+        connect(this, &Group::groupAboutToAdd, db, &Database::groupAboutToAdd);
+        connect(this, &Group::groupAdded, db, &Database::groupAdded);
+        connect(this, &Group::aboutToMove, db, &Database::groupAboutToMove);
+        connect(this, &Group::groupMoved, db, &Database::groupMoved);
+        connect(this, &Group::groupNonDataChange, db, &Database::markNonDataChange);
+        connect(this, &Group::modified, db, &Database::markAsModified);
         // clang-format on
     }
 
@@ -1020,7 +1045,7 @@ void Group::cleanupParent()
     if (m_parent) {
         emit groupAboutToRemove(this);
         m_parent->m_children.removeAll(this);
-        emit groupModified();
+        emitModified();
         emit groupRemoved();
     }
 }
@@ -1077,30 +1102,6 @@ bool Group::resolveAutoTypeEnabled() const
     }
 }
 
-QStringList Group::locate(const QString& locateTerm, const QString& currentPath) const
-{
-    // TODO: Replace with EntrySearcher
-    QStringList response;
-    if (locateTerm.isEmpty()) {
-        return response;
-    }
-
-    for (const Entry* entry : asConst(m_entries)) {
-        QString entryPath = currentPath + entry->title();
-        if (entryPath.contains(locateTerm, Qt::CaseInsensitive)) {
-            response << entryPath;
-        }
-    }
-
-    for (const Group* group : asConst(m_children)) {
-        for (const QString& path : group->locate(locateTerm, currentPath + group->name() + QString("/"))) {
-            response << path;
-        }
-    }
-
-    return response;
-}
-
 Entry* Group::addEntryWithPath(const QString& entryPath)
 {
     if (entryPath.isEmpty() || findEntryByPath(entryPath)) {
@@ -1132,7 +1133,8 @@ void Group::applyGroupIconOnCreateTo(Entry* entry)
         return;
     }
 
-    if (iconNumber() == Group::DefaultIconNumber && iconUuid().isNull()) {
+    if ((iconNumber() == Group::DefaultIconNumber || iconNumber() == Group::OpenFolderIconNumber)
+        && iconUuid().isNull()) {
         return;
     }
 
@@ -1177,19 +1179,47 @@ void Group::applyGroupIconToChildEntries()
 
 void Group::sortChildrenRecursively(bool reverse)
 {
-    std::sort(
-        m_children.begin(), m_children.end(), [reverse](const Group* childGroup1, const Group* childGroup2) -> bool {
-            QString name1 = childGroup1->name();
-            QString name2 = childGroup2->name();
-            return reverse ? name1.compare(name2, Qt::CaseInsensitive) > 0
-                           : name1.compare(name2, Qt::CaseInsensitive) < 0;
-        });
+    Group* recycleBin = nullptr;
+    if (database()) {
+        recycleBin = database()->metadata()->recycleBin();
+    }
+    std::sort(m_children.begin(), m_children.end(), [=](const Group* childGroup1, const Group* childGroup2) -> bool {
+        if (childGroup1 == recycleBin) {
+            return false;
+        }
+        QString name1 = childGroup1->name();
+        QString name2 = childGroup2->name();
+        return reverse ? name1.compare(name2, Qt::CaseInsensitive) > 0 : name1.compare(name2, Qt::CaseInsensitive) < 0;
+    });
 
     for (auto child : m_children) {
         child->sortChildrenRecursively(reverse);
     }
 
-    emit groupModified();
+    emitModified();
+}
+
+const Group* Group::previousParentGroup() const
+{
+    if (!database() || !database()->rootGroup()) {
+        return nullptr;
+    }
+    return database()->rootGroup()->findGroupByUuid(m_data.previousParentGroupUuid);
+}
+
+QUuid Group::previousParentGroupUuid() const
+{
+    return m_data.previousParentGroupUuid;
+}
+
+void Group::setPreviousParentGroupUuid(const QUuid& uuid)
+{
+    set(m_data.previousParentGroupUuid, uuid);
+}
+
+void Group::setPreviousParentGroup(const Group* group)
+{
+    setPreviousParentGroupUuid(group ? group->uuid() : QUuid());
 }
 
 bool Group::GroupData::operator==(const Group::GroupData& other) const
@@ -1210,6 +1240,9 @@ bool Group::GroupData::equals(const Group::GroupData& other, CompareItemOptions
     if (::compare(notes, other.notes, options) != 0) {
         return false;
     }
+    if (::compare(tags, other.tags, options) != 0) {
+        return false;
+    }
     if (::compare(iconNumber, other.iconNumber) != 0) {
         return false;
     }
