@@ -21,29 +21,29 @@
 #include "fdosecrets/objects/Item.h"
 #include "fdosecrets/objects/Prompt.h"
 #include "fdosecrets/objects/Service.h"
-#include "fdosecrets/objects/Session.h"
 
-#include "core/Config.h"
-#include "core/Database.h"
 #include "core/Tools.h"
-#include "gui/DatabaseTabWidget.h"
 #include "gui/DatabaseWidget.h"
+#include "gui/GuiTools.h"
 
+#include <QEventLoop>
 #include <QFileInfo>
-#include <QRegularExpression>
 
 namespace FdoSecrets
 {
+    Collection* Collection::Create(Service* parent, DatabaseWidget* backend)
+    {
+        return new Collection(parent, backend);
+    }
 
     Collection::Collection(Service* parent, DatabaseWidget* backend)
         : DBusObject(parent)
         , m_backend(backend)
         , m_exposedGroup(nullptr)
-        , m_registered(false)
     {
         // whenever the file path or the database object itself change, we do a full reload.
-        connect(backend, &DatabaseWidget::databaseFilePathChanged, this, &Collection::reloadBackend);
-        connect(backend, &DatabaseWidget::databaseReplaced, this, &Collection::reloadBackend);
+        connect(backend, &DatabaseWidget::databaseFilePathChanged, this, &Collection::reloadBackendOrDelete);
+        connect(backend, &DatabaseWidget::databaseReplaced, this, &Collection::reloadBackendOrDelete);
 
         // also remember to clear/populate the database when lock state changes.
         connect(backend, &DatabaseWidget::databaseUnlocked, this, &Collection::onDatabaseLockChanged);
@@ -56,37 +56,27 @@ namespace FdoSecrets
             }
             emit doneUnlockCollection(accepted);
         });
-
-        reloadBackend();
     }
 
-    void Collection::reloadBackend()
+    bool Collection::reloadBackend()
     {
-        if (m_registered) {
-            // delete all items
-            // this has to be done because the backend is actually still there, just we don't expose them
-            // NOTE: Do NOT use a for loop, because Item::doDelete will remove itself from m_items.
-            while (!m_items.isEmpty()) {
-                m_items.first()->doDelete();
-            }
-            cleanupConnections();
-
-            unregisterCurrentPath();
-            m_registered = false;
-        }
-
         Q_ASSERT(m_backend);
 
-        // make sure we have updated copy of the filepath, which is used to identify the database.
-        m_backendPath = m_backend->database()->filePath();
+        // delete all items
+        // this has to be done because the backend is actually still there, just we don't expose them
+        // NOTE: Do NOT use a for loop, because Item::removeFromDBus will remove itself from m_items.
+        while (!m_items.isEmpty()) {
+            m_items.first()->removeFromDBus();
+        }
+        cleanupConnections();
+        dbus()->unregisterObject(this);
 
-        // the database may not have a name (derived from filePath) yet, which may happen if it's newly created.
-        // defer the registration to next time a file path change happens.
-        if (!name().isEmpty()) {
-            registerWithPath(
-                QStringLiteral(DBUS_PATH_TEMPLATE_COLLECTION).arg(p()->objectPath().path(), encodePath(name())),
-                new CollectionAdaptor(this));
-            m_registered = true;
+        // make sure we have updated copy of the filepath, which is used to identify the database.
+        m_backendPath = m_backend->database()->canonicalFilePath();
+
+        // register the object, handling potentially duplicated name
+        if (!dbus()->registerObject(this)) {
+            return false;
         }
 
         // populate contents after expose on dbus, because items rely on parent's dbus object path
@@ -95,54 +85,67 @@ namespace FdoSecrets
         } else {
             cleanupConnections();
         }
+
+        emit collectionChanged();
+        return true;
     }
 
-    DBusReturn<void> Collection::ensureBackend() const
+    void Collection::reloadBackendOrDelete()
+    {
+        if (!reloadBackend()) {
+            removeFromDBus();
+        }
+    }
+
+    DBusResult Collection::ensureBackend() const
     {
         if (!m_backend) {
-            return DBusReturn<>::Error(QStringLiteral(DBUS_ERROR_SECRET_NO_SUCH_OBJECT));
+            return DBusResult(DBUS_ERROR_SECRET_NO_SUCH_OBJECT);
         }
         return {};
     }
 
-    DBusReturn<void> Collection::ensureUnlocked() const
+    DBusResult Collection::ensureUnlocked() const
     {
         if (backendLocked()) {
-            return DBusReturn<>::Error(QStringLiteral(DBUS_ERROR_SECRET_IS_LOCKED));
+            return DBusResult(DBUS_ERROR_SECRET_IS_LOCKED);
         }
         return {};
     }
 
-    DBusReturn<const QList<Item*>> Collection::items() const
+    DBusResult Collection::items(QList<Item*>& items) const
     {
         auto ret = ensureBackend();
-        if (ret.isError()) {
+        if (ret.err()) {
             return ret;
         }
-        return m_items;
+        items = m_items;
+        return {};
     }
 
-    DBusReturn<QString> Collection::label() const
+    DBusResult Collection::label(QString& label) const
     {
         auto ret = ensureBackend();
-        if (ret.isError()) {
+        if (ret.err()) {
             return ret;
         }
 
         if (backendLocked()) {
-            return name();
+            label = name();
+        } else {
+            label = m_backend->database()->metadata()->name();
         }
-        return m_backend->database()->metadata()->name();
+        return {};
     }
 
-    DBusReturn<void> Collection::setLabel(const QString& label)
+    DBusResult Collection::setLabel(const QString& label)
     {
         auto ret = ensureBackend();
-        if (ret.isError()) {
+        if (ret.err()) {
             return ret;
         }
         ret = ensureUnlocked();
-        if (ret.isError()) {
+        if (ret.err()) {
             return ret;
         }
 
@@ -150,78 +153,77 @@ namespace FdoSecrets
         return {};
     }
 
-    DBusReturn<bool> Collection::locked() const
+    DBusResult Collection::locked(bool& locked) const
     {
         auto ret = ensureBackend();
-        if (ret.isError()) {
+        if (ret.err()) {
             return ret;
         }
-        return backendLocked();
+        locked = backendLocked();
+        return {};
     }
 
-    DBusReturn<qulonglong> Collection::created() const
+    DBusResult Collection::created(qulonglong& created) const
     {
         auto ret = ensureBackend();
-        if (ret.isError()) {
+        if (ret.err()) {
             return ret;
         }
         ret = ensureUnlocked();
-        if (ret.isError()) {
+        if (ret.err()) {
             return ret;
         }
-        return static_cast<qulonglong>(m_backend->database()->rootGroup()->timeInfo().creationTime().toMSecsSinceEpoch()
-                                       / 1000);
+        created = static_cast<qulonglong>(
+            m_backend->database()->rootGroup()->timeInfo().creationTime().toMSecsSinceEpoch() / 1000);
+
+        return {};
     }
 
-    DBusReturn<qulonglong> Collection::modified() const
+    DBusResult Collection::modified(qulonglong& modified) const
     {
         auto ret = ensureBackend();
-        if (ret.isError()) {
+        if (ret.err()) {
             return ret;
         }
         ret = ensureUnlocked();
-        if (ret.isError()) {
+        if (ret.err()) {
             return ret;
         }
         // FIXME: there seems not to have a global modified time.
         // Use a more accurate time, considering all metadata, group, entry.
-        return static_cast<qulonglong>(
+        modified = static_cast<qulonglong>(
             m_backend->database()->rootGroup()->timeInfo().lastModificationTime().toMSecsSinceEpoch() / 1000);
+        return {};
     }
 
-    DBusReturn<PromptBase*> Collection::deleteCollection()
+    DBusResult Collection::remove(PromptBase*& prompt)
     {
         auto ret = ensureBackend();
-        if (ret.isError()) {
+        if (ret.err()) {
+            return ret;
+        }
+
+        prompt = PromptBase::Create<DeleteCollectionPrompt>(service(), this);
+        if (!prompt) {
+            return QDBusError::InternalError;
+        }
+        return {};
+    }
+
+    DBusResult Collection::searchItems(const DBusClientPtr&, const StringStringMap& attributes, QList<Item*>& items)
+    {
+        items.clear();
+
+        auto ret = ensureBackend();
+        if (ret.err()) {
             return ret;
         }
 
-        // Delete means close database
-        auto prompt = new DeleteCollectionPrompt(service(), this);
         if (backendLocked()) {
-            // this won't raise a dialog, immediate execute
-            auto pret = prompt->prompt({});
-            if (pret.isError()) {
-                return pret;
-            }
-            prompt = nullptr;
-        }
-        // defer the close to the prompt
-        return prompt;
-    }
-
-    DBusReturn<const QList<Item*>> Collection::searchItems(const StringStringMap& attributes)
-    {
-        auto ret = ensureBackend();
-        if (ret.isError()) {
-            return ret;
-        }
-        ret = ensureUnlocked();
-        if (ret.isError()) {
             // searchItems should work, whether `this` is locked or not.
             // however, we can't search items the same way as in gnome-keying,
             // because there's no database at all when locked.
-            return QList<Item*>{};
+            return {};
         }
 
         // shortcut logic for Uuid/Path attributes, as they can uniquely identify an item.
@@ -229,20 +231,18 @@ namespace FdoSecrets
             auto uuid = QUuid::fromRfc4122(QByteArray::fromHex(attributes.value(ItemAttributes::UuidKey).toLatin1()));
             auto entry = m_exposedGroup->findEntryByUuid(uuid);
             if (entry) {
-                return QList<Item*>{m_entryToItem.value(entry)};
-            } else {
-                return QList<Item*>{};
+                items += m_entryToItem.value(entry);
             }
+            return {};
         }
 
         if (attributes.contains(ItemAttributes::PathKey)) {
             auto path = attributes.value(ItemAttributes::PathKey);
             auto entry = m_exposedGroup->findEntryByPath(path);
             if (entry) {
-                return QList<Item*>{m_entryToItem.value(entry)};
-            } else {
-                return QList<Item*>{};
+                items += m_entryToItem.value(entry);
             }
+            return {};
         }
 
         QList<EntrySearcher::SearchTerm> terms;
@@ -250,13 +250,21 @@ namespace FdoSecrets
             terms << attributeToTerm(it.key(), it.value());
         }
 
-        QList<Item*> items;
-        const auto foundEntries = EntrySearcher(false, true).search(terms, m_exposedGroup);
+        constexpr auto caseSensitive = false;
+        constexpr auto skipProtected = true;
+        constexpr auto forceSearch = true;
+        const auto foundEntries =
+            EntrySearcher(caseSensitive, skipProtected).search(terms, m_exposedGroup, forceSearch);
         items.reserve(foundEntries.size());
         for (const auto& entry : foundEntries) {
-            items << m_entryToItem.value(entry);
+            const auto item = m_entryToItem.value(entry);
+            // it's possible that we don't have a corresponding item for the entry
+            // this can happen when the recycle bin is below the exposed group.
+            if (item) {
+                items << item;
+            }
         }
-        return items;
+        return {};
     }
 
     EntrySearcher::SearchTerm Collection::attributeToTerm(const QString& key, const QString& value)
@@ -272,108 +280,40 @@ namespace FdoSecrets
         term.field = attrKeyToField.value(key, EntrySearcher::Field::AttributeValue);
         term.word = key;
         term.exclude = false;
-
-        const auto useWildcards = false;
-        const auto exactMatch = true;
-        const auto caseSensitive = true;
-        term.regex = Tools::convertToRegex(QRegularExpression::escape(value), useWildcards, exactMatch, caseSensitive);
+        term.regex =
+            Tools::convertToRegex(value,
+                                  Tools::RegexConvertOpts::EXACT_MATCH | Tools::RegexConvertOpts::CASE_SENSITIVE
+                                      | Tools::RegexConvertOpts::ESCAPE_REGEX);
 
         return term;
     }
 
-    DBusReturn<Item*>
-    Collection::createItem(const QVariantMap& properties, const SecretStruct& secret, bool replace, PromptBase*& prompt)
+    DBusResult Collection::createItem(const QVariantMap& properties,
+                                      const Secret& secret,
+                                      bool replace,
+                                      Item*& item,
+                                      PromptBase*& prompt)
     {
         auto ret = ensureBackend();
-        if (ret.isError()) {
-            return ret;
-        }
-        ret = ensureUnlocked();
-        if (ret.isError()) {
+        if (ret.err()) {
             return ret;
         }
 
-        if (!pathToObject<Session>(secret.session)) {
-            return DBusReturn<>::Error(QStringLiteral(DBUS_ERROR_SECRET_NO_SESSION));
+        item = nullptr;
+
+        prompt = PromptBase::Create<CreateItemPrompt>(service(), this, properties, secret, replace);
+        if (!prompt) {
+            return QDBusError::InternalError;
         }
-
-        prompt = nullptr;
-
-        bool newlyCreated = true;
-        Item* item = nullptr;
-        QString itemPath;
-        StringStringMap attributes;
-
-        auto iterAttr = properties.find(QStringLiteral(DBUS_INTERFACE_SECRET_ITEM ".Attributes"));
-        if (iterAttr != properties.end()) {
-            attributes = iterAttr.value().value<StringStringMap>();
-
-            itemPath = attributes.value(ItemAttributes::PathKey);
-
-            // check existing item using attributes
-            auto existings = searchItems(attributes);
-            if (existings.isError()) {
-                return existings;
-            }
-            if (!existings.value().isEmpty() && replace) {
-                item = existings.value().front();
-                newlyCreated = false;
-            }
-        }
-
-        if (!item) {
-            // normalize itemPath
-            itemPath = itemPath.startsWith('/') ? QString{} : QStringLiteral("/") + itemPath;
-
-            // split itemPath to groupPath and itemName
-            auto components = itemPath.split('/');
-            Q_ASSERT(components.size() >= 2);
-
-            auto itemName = components.takeLast();
-            Group* group = findCreateGroupByPath(components.join('/'));
-
-            // create new Entry in backend
-            auto* entry = new Entry();
-            entry->setUuid(QUuid::createUuid());
-            entry->setTitle(itemName);
-            entry->setUsername(m_backend->database()->metadata()->defaultUserName());
-            group->applyGroupIconOnCreateTo(entry);
-
-            entry->setGroup(group);
-
-            // when creation finishes in backend, we will already have item
-            item = m_entryToItem.value(entry, nullptr);
-
-            if (!item) {
-                // may happen if entry somehow ends up in recycle bin
-                return DBusReturn<>::Error(QStringLiteral(DBUS_ERROR_SECRET_NO_SUCH_OBJECT));
-            }
-        }
-
-        ret = item->setProperties(properties);
-        if (ret.isError()) {
-            if (newlyCreated) {
-                item->doDelete();
-            }
-            return ret;
-        }
-        ret = item->setSecret(secret);
-        if (ret.isError()) {
-            if (newlyCreated) {
-                item->doDelete();
-            }
-            return ret;
-        }
-
-        return item;
+        return {};
     }
 
-    DBusReturn<void> Collection::setProperties(const QVariantMap& properties)
+    DBusResult Collection::setProperties(const QVariantMap& properties)
     {
-        auto label = properties.value(QStringLiteral(DBUS_INTERFACE_SECRET_COLLECTION ".Label")).toString();
+        auto label = properties.value(DBUS_INTERFACE_SECRET_COLLECTION + ".Label").toString();
 
         auto ret = setLabel(label);
-        if (ret.isError()) {
+        if (ret.err()) {
             return ret;
         }
 
@@ -385,10 +325,10 @@ namespace FdoSecrets
         return m_aliases;
     }
 
-    DBusReturn<void> Collection::addAlias(QString alias)
+    DBusResult Collection::addAlias(QString alias)
     {
         auto ret = ensureBackend();
-        if (ret.isError()) {
+        if (ret.err()) {
             return ret;
         }
 
@@ -400,20 +340,20 @@ namespace FdoSecrets
 
         emit aliasAboutToAdd(alias);
 
-        bool ok = QDBusConnection::sessionBus().registerObject(
-            QStringLiteral(DBUS_PATH_TEMPLATE_ALIAS).arg(p()->objectPath().path(), alias), this);
-        if (ok) {
+        if (dbus()->registerAlias(this, alias)) {
             m_aliases.insert(alias);
             emit aliasAdded(alias);
+        } else {
+            return QDBusError::InvalidObjectPath;
         }
 
         return {};
     }
 
-    DBusReturn<void> Collection::removeAlias(QString alias)
+    DBusResult Collection::removeAlias(QString alias)
     {
         auto ret = ensureBackend();
-        if (ret.isError()) {
+        if (ret.err()) {
             return ret;
         }
 
@@ -423,9 +363,7 @@ namespace FdoSecrets
             return {};
         }
 
-        QDBusConnection::sessionBus().unregisterObject(
-            QStringLiteral(DBUS_PATH_TEMPLATE_ALIAS).arg(p()->objectPath().path(), alias));
-
+        dbus()->unregisterAlias(alias);
         m_aliases.remove(alias);
         emit aliasRemoved(alias);
 
@@ -435,16 +373,15 @@ namespace FdoSecrets
     QString Collection::name() const
     {
         if (m_backendPath.isEmpty()) {
-            // This is a newly created db without saving to file.
-            // This name is also used to register dbus path.
-            // For simplicity, we don't monitor the name change.
-            // So the dbus object path is not updated if the db name
-            // changes. This should not be a problem because once the database
-            // gets saved, the dbus path will be updated to use filename and
-            // everything back to normal.
+            // This is a newly created db without saving to file,
+            // but we have to give a name, which is used to register dbus path.
+            // We use database name for this purpose. For simplicity, we don't monitor the name change.
+            // So the dbus object path is not updated if the db name changes.
+            // This should not be a problem because once the database gets saved,
+            // the dbus path will be updated to use filename and everything back to normal.
             return m_backend->database()->metadata()->name();
         }
-        return QFileInfo(m_backendPath).baseName();
+        return QFileInfo(m_backendPath).completeBaseName();
     }
 
     DatabaseWidget* Collection::backend() const
@@ -454,19 +391,16 @@ namespace FdoSecrets
 
     void Collection::onDatabaseLockChanged()
     {
-        auto locked = backendLocked();
-        if (!locked) {
-            populateContents();
-        } else {
-            cleanupConnections();
+        if (!reloadBackend()) {
+            removeFromDBus();
+            return;
         }
-        emit collectionLockChanged(locked);
-        emit collectionChanged();
+        emit collectionLockChanged(backendLocked());
     }
 
     void Collection::populateContents()
     {
-        if (!m_registered) {
+        if (!m_backend) {
             return;
         }
 
@@ -476,7 +410,7 @@ namespace FdoSecrets
         auto newGroup = m_backend->database()->rootGroup()->findGroupByUuid(newUuid);
         if (!newGroup || inRecycleBin(newGroup)) {
             // no exposed group, delete self
-            doDelete();
+            removeFromDBus();
             return;
         }
 
@@ -509,15 +443,15 @@ namespace FdoSecrets
             }
         });
         // Another possibility is the group being moved to recycle bin.
-        connect(m_exposedGroup.data(), &Group::groupModified, this, [this]() {
-            if (inRecycleBin(m_exposedGroup->parentGroup())) {
+        connect(m_exposedGroup.data(), &Group::modified, this, [this]() {
+            if (inRecycleBin(m_exposedGroup)) {
                 // reset the exposed group to none
                 FdoSecrets::settings()->setExposedGroup(m_backend->database().data(), {});
             }
         });
 
         // Monitor exposed group settings
-        connect(m_backend->database()->metadata()->customData(), &CustomData::customDataModified, this, [this]() {
+        connect(m_backend->database()->metadata()->customData(), &CustomData::modified, this, [this]() {
             if (!m_exposedGroup || backendLocked()) {
                 return;
             }
@@ -534,6 +468,8 @@ namespace FdoSecrets
             onEntryAdded(entry, false);
         }
 
+        // Do not connect to Database::modified signal because we only want signals for the subset under m_exposedGroup
+        connect(m_backend->database()->metadata(), &Metadata::modified, this, &Collection::collectionChanged);
         connectGroupSignalRecursive(m_exposedGroup);
     }
 
@@ -543,7 +479,7 @@ namespace FdoSecrets
         // this has to be done because the backend is actually still there
         // just we don't expose them
         for (const auto& item : asConst(m_items)) {
-            item->doDelete();
+            item->removeFromDBus();
         }
 
         // repopulate
@@ -558,14 +494,14 @@ namespace FdoSecrets
             return;
         }
 
-        auto item = new Item(this, entry);
+        auto item = Item::Create(this, entry);
         m_items << item;
         m_entryToItem[entry] = item;
 
         // forward delete signals
         connect(entry->group(), &Group::entryAboutToRemove, item, [item](Entry* toBeRemoved) {
             if (item->backend() == toBeRemoved) {
-                item->doDelete();
+                item->removeFromDBus();
             }
         });
 
@@ -588,7 +524,7 @@ namespace FdoSecrets
             return;
         }
 
-        connect(group, &Group::groupModified, this, &Collection::collectionChanged);
+        connect(group, &Group::modified, this, &Collection::collectionChanged);
         connect(group, &Group::entryAdded, this, [this](Entry* entry) { onEntryAdded(entry, true); });
 
         const auto children = group->children();
@@ -606,17 +542,26 @@ namespace FdoSecrets
     {
         Q_ASSERT(m_backend);
 
-        return m_backend->lock();
+        // do not call m_backend->lock() directly
+        // let the service handle locking which handles concurrent calls
+        return service()->doLockDatabase(m_backend);
     }
 
     void Collection::doUnlock()
     {
         Q_ASSERT(m_backend);
 
-        service()->doUnlockDatabaseInDialog(m_backend);
+        return service()->doUnlockDatabaseInDialog(m_backend);
     }
 
-    void Collection::doDelete()
+    bool Collection::doDelete()
+    {
+        Q_ASSERT(m_backend);
+
+        return service()->doCloseDatabase(m_backend);
+    }
+
+    void Collection::removeFromDBus()
     {
         if (!m_backend) {
             // I'm already deleted
@@ -625,7 +570,8 @@ namespace FdoSecrets
 
         emit collectionAboutToDelete();
 
-        unregisterCurrentPath();
+        // remove from dbus early
+        dbus()->unregisterObject(this);
 
         // remove alias manually to trigger signal
         for (const auto& a : aliases()) {
@@ -643,6 +589,8 @@ namespace FdoSecrets
         // reset backend and delete self
         m_backend = nullptr;
         deleteLater();
+
+        // items will be removed automatically as they are children objects
     }
 
     void Collection::cleanupConnections()
@@ -672,9 +620,18 @@ namespace FdoSecrets
         return !m_backend || !m_backend->database()->isInitialized() || m_backend->isLocked();
     }
 
-    void Collection::doDeleteEntries(QList<Entry*> entries)
+    bool Collection::doDeleteEntry(Entry* entry)
     {
-        m_backend->deleteEntries(std::move(entries));
+        // Confirm entry removal before moving forward
+        bool permanent = inRecycleBin(entry) || !m_backend->database()->metadata()->recycleBinEnabled();
+        if (FdoSecrets::settings()->confirmDeleteItem()
+            && !GuiTools::confirmDeleteEntries(m_backend, {entry}, permanent)) {
+            return false;
+        }
+
+        auto num = GuiTools::deleteEntriesResolveReferences(m_backend, {entry}, permanent);
+
+        return num != 0;
     }
 
     Group* Collection::findCreateGroupByPath(const QString& groupPath)
@@ -706,11 +663,7 @@ namespace FdoSecrets
     bool Collection::inRecycleBin(Group* group) const
     {
         Q_ASSERT(m_backend);
-
-        if (!group) {
-            // the root group's parent is nullptr, we treat it as not in recycle bin.
-            return false;
-        }
+        Q_ASSERT(group);
 
         if (!m_backend->database()->metadata()) {
             return false;
@@ -730,4 +683,36 @@ namespace FdoSecrets
         return inRecycleBin(entry->group());
     }
 
+    Item* Collection::doNewItem(const DBusClientPtr& client, QString itemPath)
+    {
+        Q_ASSERT(m_backend);
+
+        // normalize itemPath
+        itemPath = (itemPath.startsWith('/') ? QString{} : QStringLiteral("/")) + itemPath;
+
+        // split itemPath to groupPath and itemName
+        auto components = itemPath.split('/');
+        Q_ASSERT(components.size() >= 2);
+
+        auto itemName = components.takeLast();
+        Group* group = findCreateGroupByPath(components.join('/'));
+
+        // create new Entry in backend
+        auto* entry = new Entry();
+        entry->setUuid(QUuid::createUuid());
+        entry->setTitle(itemName);
+        entry->setUsername(m_backend->database()->metadata()->defaultUserName());
+        group->applyGroupIconOnCreateTo(entry);
+
+        entry->setGroup(group);
+
+        // the item was just created so there is no point in having it not authorized
+        client->setItemAuthorized(entry->uuid(), AuthDecision::Allowed);
+
+        // when creation finishes in backend, we will already have item
+        auto created = m_entryToItem.value(entry, nullptr);
+
+        return created;
+    }
+
 } // namespace FdoSecrets
