@@ -20,21 +20,9 @@
 
 #include "gui/MessageWidget.h"
 
-#include <QScopedPointer>
 #include <QTimer>
-#include <QWidget>
 
 class QAbstractItemView;
-class QItemEditorCreatorBase;
-class QItemEditorFactory;
-
-namespace FdoSecrets
-{
-
-    class Session;
-    class Collection;
-
-} // namespace FdoSecrets
 
 class FdoSecretsPlugin;
 
@@ -61,12 +49,8 @@ protected:
     void showEvent(QShowEvent* event) override;
     void hideEvent(QHideEvent* event) override;
 
-private:
-    void setupView(QAbstractItemView* view, int manageColumn, int editorTypeId, QItemEditorCreatorBase* creator);
-
 private:
     QScopedPointer<Ui::SettingsWidgetFdoSecrets> m_ui;
-    QScopedPointer<QItemEditorFactory> m_factory;
     FdoSecretsPlugin* m_plugin;
     QTimer m_checkTimer;
 };
