@@ -19,10 +19,7 @@
 #define KEEPASSX_CLONEDIALOG_H
 
 #include "core/Database.h"
-#include "core/Entry.h"
 #include "gui/DatabaseWidget.h"
-#include <QDialog>
-#include <QScopedPointer>
 
 namespace Ui
 {
@@ -37,6 +34,9 @@ public:
     explicit CloneDialog(DatabaseWidget* parent = nullptr, Database* db = nullptr, Entry* entry = nullptr);
     ~CloneDialog();
 
+signals:
+    void entryCloned(Entry* clone);
+
 private:
     QScopedPointer<Ui::CloneDialog> m_ui;
 
