@@ -244,6 +244,7 @@ void DatabaseTabWidget::addDatabaseTab(DatabaseWidget* dbWidget, bool inBackgrou
             SLOT(updateTabName()));
     connect(dbWidget, SIGNAL(databaseModified()), SLOT(updateTabName()));
     connect(dbWidget, SIGNAL(databaseSaved()), SLOT(updateTabName()));
+    connect(dbWidget, SIGNAL(databaseSaved()), SLOT(updateLastDatabases()));
     connect(dbWidget, SIGNAL(databaseUnlocked()), SLOT(updateTabName()));
     connect(dbWidget, SIGNAL(databaseUnlocked()), SLOT(emitDatabaseLockChanged()));
     connect(dbWidget, SIGNAL(databaseLocked()), SLOT(updateTabName()));
@@ -829,6 +830,18 @@ void DatabaseTabWidget::updateLastDatabases(const QString& filename)
     }
 }
 
+void DatabaseTabWidget::updateLastDatabases()
+{
+    auto dbWidget = currentDatabaseWidget();
+
+    if (dbWidget) {
+        auto filePath = dbWidget->database()->filePath();
+        if (!filePath.isEmpty()) {
+            updateLastDatabases(filePath);
+        }
+    }
+}
+
 void DatabaseTabWidget::emitActiveDatabaseChanged()
 {
     emit activeDatabaseChanged(currentDatabaseWidget());
