@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2018 KeePassXC Team <team@keepassxc.org>
+ *  Copyright (C) 2023 KeePassXC Team <team@keepassxc.org>
  *  Copyright (C) 2010 Felix Geyer <debfx@fobos.de>
  *
  *  This program is free software: you can redistribute it and/or modify
@@ -20,9 +20,8 @@
 #define KEEPASSX_DATABASEWIDGET_H
 
 #include <QFileSystemWatcher>
-#include <QScopedPointer>
+#include <QListView>
 #include <QStackedWidget>
-#include <QTimer>
 
 #include "DatabaseOpenDialog.h"
 #include "config-keepassx.h"
@@ -50,6 +49,8 @@ class QSplitter;
 class QLabel;
 class MessageWidget;
 class EntryPreviewWidget;
+class TagView;
+class ElidedLabel;
 
 namespace Ui
 {
@@ -74,7 +75,7 @@ public:
 
     explicit DatabaseWidget(QSharedPointer<Database> db, QWidget* parent = nullptr);
     explicit DatabaseWidget(const QString& filePath, QWidget* parent = nullptr);
-    ~DatabaseWidget();
+    ~DatabaseWidget() override;
 
     void setFocus(Qt::FocusReason reason);
 
@@ -96,6 +97,7 @@ public:
     EntryView* entryView();
 
     Group* currentGroup() const;
+    bool canCloneCurrentGroup() const;
     bool canDeleteCurrentGroup() const;
     bool isGroupSelected() const;
     bool isRecycleBinSelected() const;
@@ -105,6 +107,7 @@ public:
     QStringList customEntryAttributes() const;
     bool isEditWidgetModified() const;
     void clearAllWidgets();
+    Entry* currentSelectedEntry();
     bool currentEntryHasTitle();
     bool currentEntryHasUsername();
     bool currentEntryHasPassword();
@@ -114,18 +117,19 @@ public:
 #ifdef WITH_XC_SSHAGENT
     bool currentEntryHasSshKey();
 #endif
+    bool currentEntryHasAutoTypeEnabled();
 
     QByteArray entryViewState() const;
     bool setEntryViewState(const QByteArray& state) const;
-    QList<int> mainSplitterSizes() const;
-    void setMainSplitterSizes(const QList<int>& sizes);
-    QList<int> previewSplitterSizes() const;
-    void setPreviewSplitterSizes(const QList<int>& sizes);
+    QHash<Config::ConfigKey, QList<int>> splitterSizes() const;
+    void setSplitterSizes(const QHash<Config::ConfigKey, QList<int>>& sizes);
+    void setSearchStringForAutoType(const QString& search);
 
 signals:
     // relayed Database signals
     void databaseFilePathChanged(const QString& oldPath, const QString& newPath);
     void databaseModified();
+    void databaseNonDataChanged();
     void databaseSaved();
     void databaseUnlocked();
     void databaseLockRequested();
@@ -147,10 +151,11 @@ signals:
     void listModeActivated();
     void searchModeAboutToActivate();
     void searchModeActivated();
-    void mainSplitterSizesChanged();
-    void previewSplitterSizesChanged();
+    void splitterSizesChanged();
     void entryViewStateChanged();
     void clearSearch();
+    void requestGlobalAutoType(const QString& search);
+    void requestSearch(const QString& search);
 
 public slots:
     bool lock();
@@ -162,7 +167,8 @@ public slots:
     void createEntry();
     void cloneEntry();
     void deleteSelectedEntries();
-    void deleteEntries(QList<Entry*> entries);
+    void restoreSelectedEntries();
+    void deleteEntries(QList<Entry*> entries, bool confirm = true);
     void focusOnEntries(bool editIfFocused = false);
     void focusOnGroups(bool editIfFocused = false);
     void moveEntryUp();
@@ -173,24 +179,30 @@ public slots:
     void copyURL();
     void copyNotes();
     void copyAttribute(QAction* action);
+    void filterByTag();
+    void setTag(QAction* action);
     void showTotp();
     void showTotpKeyQrCode();
     void copyTotp();
+    void copyPasswordTotp();
     void setupTotp();
 #ifdef WITH_XC_SSHAGENT
     void addToAgent();
     void removeFromAgent();
 #endif
-    void performAutoType();
+    void performAutoType(const QString& sequence = {});
     void performAutoTypeUsername();
     void performAutoTypeUsernameEnter();
     void performAutoTypePassword();
     void performAutoTypePasswordEnter();
+    void performAutoTypeTOTP();
     void openUrl();
     void downloadSelectedFavicons();
     void downloadAllFavicons();
+    void downloadFaviconInBackground(Entry* entry);
     void openUrlForEntry(Entry* entry);
     void createGroup();
+    void cloneGroup();
     void deleteGroup();
     void switchToMainView(bool previousDialogAccepted = false);
     void switchToEntryEdit();
@@ -212,6 +224,8 @@ public slots:
 
     // Search related slots
     void search(const QString& searchtext);
+    void saveSearch(const QString& searchtext);
+    void deleteSearch(const QString& name);
     void setSearchCaseSensitive(bool state);
     void setSearchLimitGroup(bool state);
     void endSearch();
@@ -240,6 +254,7 @@ private slots:
     void onEntryChanged(Entry* entry);
     void onGroupChanged();
     void onDatabaseModified();
+    void onDatabaseNonDataChanged();
     void connectDatabaseSignals();
     void loadDatabase(bool accepted);
     void unlockDatabase(bool accepted);
@@ -254,20 +269,19 @@ private:
     void setClipboardTextAndMinimize(const QString& text);
     void processAutoOpen();
     void openDatabaseFromEntry(const Entry* entry, bool inBackground = true);
-    bool confirmDeleteEntries(QList<Entry*> entries, bool permanent);
-    void performIconDownloads(const QList<Entry*>& entries, bool force = false);
+    void performIconDownloads(const QList<Entry*>& entries, bool force = false, bool downloadInBackground = false);
     bool performSave(QString& errorMessage, const QString& fileName = {});
-    Entry* currentSelectedEntry();
 
     QSharedPointer<Database> m_db;
 
     QPointer<QWidget> m_mainWidget;
     QPointer<QSplitter> m_mainSplitter;
+    QPointer<QSplitter> m_groupSplitter;
     QPointer<MessageWidget> m_messageWidget;
     QPointer<EntryPreviewWidget> m_previewView;
     QPointer<QSplitter> m_previewSplitter;
     QPointer<QLabel> m_searchingLabel;
-    QPointer<QLabel> m_shareLabel;
+    QPointer<ElidedLabel> m_shareLabel;
     QPointer<CsvImportWizard> m_csvImportWizard;
     QPointer<EditEntryWidget> m_editEntryWidget;
     QPointer<EditGroupWidget> m_editGroupWidget;
@@ -278,6 +292,7 @@ private:
     QPointer<KeePass1OpenWidget> m_keepass1OpenWidget;
     QPointer<OpVaultOpenWidget> m_opVaultOpenWidget;
     QPointer<GroupView> m_groupView;
+    QPointer<TagView> m_tagView;
     QPointer<EntryView> m_entryView;
 
     QScopedPointer<Group> m_newGroup;
@@ -290,12 +305,16 @@ private:
     int m_saveAttempts;
 
     // Search state
-    EntrySearcher* m_EntrySearcher;
+    QScopedPointer<EntrySearcher> m_entrySearcher;
     QString m_lastSearchText;
+    QString m_nextSearchLabelText;
     bool m_searchLimitGroup;
 
     // Autoreload
     bool m_blockAutoSave;
+
+    // Auto-Type related
+    QString m_searchStringForAutoType;
 };
 
 #endif // KEEPASSX_DATABASEWIDGET_H
