@@ -1,6 +1,6 @@
 /*
+ *  Copyright (C) 2023 KeePassXC Team <team@keepassxc.org>
  *  Copyright (C) 2010 Felix Geyer <debfx@fobos.de>
- *  Copyright (C) 2020 KeePassXC Team <team@keepassxc.org>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 
 #include <QCloseEvent>
 #include <QDesktopServices>
+#include <QDir>
 #include <QFileInfo>
 #include <QList>
 #include <QMimeData>
@@ -42,6 +43,7 @@
 #include "gui/Icons.h"
 #include "gui/MessageBox.h"
 #include "gui/SearchWidget.h"
+#include "gui/entry/EntryView.h"
 #include "gui/osutils/OSUtils.h"
 
 #ifdef WITH_XC_UPDATECHECK
@@ -126,8 +128,10 @@ MainWindow::MainWindow()
     m_entryContextMenu->setSeparatorsCollapsible(true);
     m_entryContextMenu->addAction(m_ui->actionEntryCopyUsername);
     m_entryContextMenu->addAction(m_ui->actionEntryCopyPassword);
+    m_entryContextMenu->addAction(m_ui->actionEntryCopyURL);
     m_entryContextMenu->addAction(m_ui->menuEntryCopyAttribute->menuAction());
     m_entryContextMenu->addAction(m_ui->menuEntryTotp->menuAction());
+    m_entryContextMenu->addAction(m_ui->menuTags->menuAction());
     m_entryContextMenu->addSeparator();
     m_entryContextMenu->addAction(m_ui->actionEntryAutoType);
     m_entryContextMenu->addSeparator();
@@ -142,6 +146,9 @@ MainWindow::MainWindow()
     m_entryContextMenu->addAction(m_ui->actionEntryOpenUrl);
     m_entryContextMenu->addAction(m_ui->actionEntryDownloadIcon);
     m_entryContextMenu->addSeparator();
+    m_entryContextMenu->addAction(m_ui->actionEntryAddToAgent);
+    m_entryContextMenu->addAction(m_ui->actionEntryRemoveFromAgent);
+    m_entryContextMenu->addSeparator();
     m_entryContextMenu->addAction(m_ui->actionEntryRestore);
 
     m_entryNewContextMenu = new QMenu(this);
@@ -188,18 +195,8 @@ MainWindow::MainWindow()
     connect(sshAgent(), SIGNAL(error(QString)), this, SLOT(showErrorMessage(QString)));
     connect(sshAgent(), SIGNAL(enabledChanged(bool)), this, SLOT(agentEnabled(bool)));
     m_ui->settingsWidget->addSettingsPage(new AgentSettingsPage());
-
-    m_entryContextMenu->addSeparator();
-    m_entryContextMenu->addAction(m_ui->actionEntryAddToAgent);
-    m_entryContextMenu->addAction(m_ui->actionEntryRemoveFromAgent);
-
-    m_ui->actionEntryAddToAgent->setIcon(icons()->icon("utilities-terminal"));
-    m_ui->actionEntryRemoveFromAgent->setIcon(icons()->icon("utilities-terminal"));
 #endif
 
-    m_ui->actionEntryAddToAgent->setVisible(false);
-    m_ui->actionEntryRemoveFromAgent->setVisible(false);
-
     initViewMenu();
 
 #if defined(WITH_XC_KEESHARE)
@@ -239,6 +236,11 @@ MainWindow::MainWindow()
         m_copyAdditionalAttributeActions, SIGNAL(triggered(QAction*)), SLOT(copyAttribute(QAction*)));
     connect(m_ui->menuEntryCopyAttribute, SIGNAL(aboutToShow()), this, SLOT(updateCopyAttributesMenu()));
 
+    m_setTagsMenuActions = new QActionGroup(m_ui->menuTags);
+    m_setTagsMenuActions->setExclusive(false);
+    m_actionMultiplexer.connect(m_setTagsMenuActions, SIGNAL(triggered(QAction*)), SLOT(setTag(QAction*)));
+    connect(m_ui->menuTags, &QMenu::aboutToShow, this, &MainWindow::updateSetTagsMenu);
+
     Qt::Key globalAutoTypeKey = static_cast<Qt::Key>(config()->get(Config::GlobalAutoTypeKey).toInt());
     Qt::KeyboardModifiers globalAutoTypeModifiers =
         static_cast<Qt::KeyboardModifiers>(config()->get(Config::GlobalAutoTypeModifiers).toInt());
@@ -250,6 +252,7 @@ MainWindow::MainWindow()
     m_showToolbarSeparator = config()->get(Config::GUI_ApplicationTheme).toString() != "classic";
 
     m_ui->actionEntryAutoType->setVisible(autoType()->isAvailable());
+    m_ui->actionAllowScreenCapture->setVisible(osUtils->canPreventScreenCapture());
 
     m_inactivityTimer = new InactivityTimer(this);
     connect(m_inactivityTimer, SIGNAL(inactivityDetected()), this, SLOT(lockDatabasesAfterInactivity()));
@@ -260,6 +263,9 @@ MainWindow::MainWindow()
     setShortcut(m_ui->actionDatabaseSave, QKeySequence::Save, Qt::CTRL + Qt::Key_S);
     setShortcut(m_ui->actionDatabaseSaveAs, QKeySequence::SaveAs, Qt::CTRL + Qt::SHIFT + Qt::Key_S);
     setShortcut(m_ui->actionDatabaseClose, QKeySequence::Close, Qt::CTRL + Qt::Key_W);
+    m_ui->actionDatabaseSettings->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_Comma);
+    m_ui->actionReports->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_R);
+    setShortcut(m_ui->actionSettings, QKeySequence::Preferences, Qt::CTRL + Qt::Key_Comma);
     m_ui->actionLockDatabase->setShortcut(Qt::CTRL + Qt::Key_L);
     m_ui->actionLockAllDatabases->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_L);
     setShortcut(m_ui->actionQuit, QKeySequence::Quit, Qt::CTRL + Qt::Key_Q);
@@ -271,10 +277,12 @@ MainWindow::MainWindow()
     m_ui->actionEntryTotp->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_T);
     m_ui->actionEntryDownloadIcon->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_D);
     m_ui->actionEntryCopyTotp->setShortcut(Qt::CTRL + Qt::Key_T);
+    m_ui->actionEntryCopyPasswordTotp->setShortcut(Qt::CTRL + Qt::Key_Y);
     m_ui->actionEntryMoveUp->setShortcut(Qt::CTRL + Qt::ALT + Qt::Key_Up);
     m_ui->actionEntryMoveDown->setShortcut(Qt::CTRL + Qt::ALT + Qt::Key_Down);
     m_ui->actionEntryCopyUsername->setShortcut(Qt::CTRL + Qt::Key_B);
     m_ui->actionEntryCopyPassword->setShortcut(Qt::CTRL + Qt::Key_C);
+    m_ui->actionEntryCopyTitle->setShortcut(Qt::CTRL + Qt::Key_I);
     m_ui->actionEntryAutoTypeSequence->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_V);
     m_ui->actionEntryOpenUrl->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_U);
     m_ui->actionEntryCopyURL->setShortcut(Qt::CTRL + Qt::Key_U);
@@ -300,6 +308,7 @@ MainWindow::MainWindow()
     m_ui->actionEntryTotp->setShortcutVisibleInContextMenu(true);
     m_ui->actionEntryDownloadIcon->setShortcutVisibleInContextMenu(true);
     m_ui->actionEntryCopyTotp->setShortcutVisibleInContextMenu(true);
+    m_ui->actionEntryCopyPasswordTotp->setShortcutVisibleInContextMenu(true);
     m_ui->actionEntryMoveUp->setShortcutVisibleInContextMenu(true);
     m_ui->actionEntryMoveDown->setShortcutVisibleInContextMenu(true);
     m_ui->actionEntryCopyUsername->setShortcutVisibleInContextMenu(true);
@@ -307,6 +316,7 @@ MainWindow::MainWindow()
     m_ui->actionEntryAutoTypeSequence->setShortcutVisibleInContextMenu(true);
     m_ui->actionEntryOpenUrl->setShortcutVisibleInContextMenu(true);
     m_ui->actionEntryCopyURL->setShortcutVisibleInContextMenu(true);
+    m_ui->actionEntryCopyTitle->setShortcutVisibleInContextMenu(true);
     m_ui->actionEntryAddToAgent->setShortcutVisibleInContextMenu(true);
     m_ui->actionEntryRemoveFromAgent->setShortcutVisibleInContextMenu(true);
 #endif
@@ -394,6 +404,16 @@ MainWindow::MainWindow()
     m_ui->actionEntryCopyUsername->setIcon(icons()->icon("username-copy"));
     m_ui->actionEntryCopyPassword->setIcon(icons()->icon("password-copy"));
     m_ui->actionEntryCopyURL->setIcon(icons()->icon("url-copy"));
+    m_ui->menuEntryCopyAttribute->setIcon(icons()->icon("attributes-copy"));
+    m_ui->menuEntryTotp->setIcon(icons()->icon("totp"));
+    m_ui->actionEntryTotp->setIcon(icons()->icon("totp"));
+    m_ui->actionEntryCopyTotp->setIcon(icons()->icon("totp-copy"));
+    m_ui->actionEntryCopyPasswordTotp->setIcon(icons()->icon("totp-copy-password"));
+    m_ui->actionEntryTotpQRCode->setIcon(icons()->icon("qrcode"));
+    m_ui->actionEntrySetupTotp->setIcon(icons()->icon("totp-edit"));
+    m_ui->actionEntryAddToAgent->setIcon(icons()->icon("utilities-terminal"));
+    m_ui->actionEntryRemoveFromAgent->setIcon(icons()->icon("utilities-terminal"));
+    m_ui->menuTags->setIcon(icons()->icon("tag-multiple"));
     m_ui->actionEntryDownloadIcon->setIcon(icons()->icon("favicon-download"));
     m_ui->actionGroupSortAsc->setIcon(icons()->icon("sort-alphabetical-ascending"));
     m_ui->actionGroupSortDesc->setIcon(icons()->icon("sort-alphabetical-descending"));
@@ -422,8 +442,14 @@ MainWindow::MainWindow()
         SIGNAL(currentModeChanged(DatabaseWidget::Mode)), this, SLOT(setMenuActionState(DatabaseWidget::Mode)));
     m_actionMultiplexer.connect(SIGNAL(groupChanged()), this, SLOT(setMenuActionState()));
     m_actionMultiplexer.connect(SIGNAL(entrySelectionChanged()), this, SLOT(setMenuActionState()));
+    m_actionMultiplexer.connect(SIGNAL(databaseNonDataChanged()), this, SLOT(setMenuActionState()));
     m_actionMultiplexer.connect(SIGNAL(groupContextMenuRequested(QPoint)), this, SLOT(showGroupContextMenu(QPoint)));
     m_actionMultiplexer.connect(SIGNAL(entryContextMenuRequested(QPoint)), this, SLOT(showEntryContextMenu(QPoint)));
+    m_actionMultiplexer.connect(SIGNAL(groupChanged()), this, SLOT(updateEntryCountLabel()));
+    m_actionMultiplexer.connect(SIGNAL(databaseUnlocked()), this, SLOT(updateEntryCountLabel()));
+    m_actionMultiplexer.connect(SIGNAL(databaseModified()), this, SLOT(updateEntryCountLabel()));
+    m_actionMultiplexer.connect(SIGNAL(searchModeActivated()), this, SLOT(updateEntryCountLabel()));
+    m_actionMultiplexer.connect(SIGNAL(listModeActivated()), this, SLOT(updateEntryCountLabel()));
 
     // Notify search when the active database changes or gets locked
     connect(m_ui->tabWidget,
@@ -462,6 +488,7 @@ MainWindow::MainWindow()
     connect(m_ui->actionImportOpVault, SIGNAL(triggered()), m_ui->tabWidget, SLOT(importOpVaultDatabase()));
     connect(m_ui->actionExportCsv, SIGNAL(triggered()), m_ui->tabWidget, SLOT(exportToCsv()));
     connect(m_ui->actionExportHtml, SIGNAL(triggered()), m_ui->tabWidget, SLOT(exportToHtml()));
+    connect(m_ui->actionExportXML, SIGNAL(triggered()), m_ui->tabWidget, SLOT(exportToXML()));
     connect(
         m_ui->actionLockDatabase, SIGNAL(triggered()), m_ui->tabWidget, SLOT(lockAndSwitchToFirstUnlockedDatabase()));
     connect(m_ui->actionLockDatabaseToolbar, SIGNAL(triggered()), m_ui->actionLockDatabase, SIGNAL(triggered()));
@@ -478,6 +505,7 @@ MainWindow::MainWindow()
     m_actionMultiplexer.connect(m_ui->actionEntrySetupTotp, SIGNAL(triggered()), SLOT(setupTotp()));
 
     m_actionMultiplexer.connect(m_ui->actionEntryCopyTotp, SIGNAL(triggered()), SLOT(copyTotp()));
+    m_actionMultiplexer.connect(m_ui->actionEntryCopyPasswordTotp, SIGNAL(triggered()), SLOT(copyPasswordTotp()));
     m_actionMultiplexer.connect(m_ui->actionEntryTotpQRCode, SIGNAL(triggered()), SLOT(showTotpKeyQrCode()));
     m_actionMultiplexer.connect(m_ui->actionEntryCopyTitle, SIGNAL(triggered()), SLOT(copyTitle()));
     m_actionMultiplexer.connect(m_ui->actionEntryMoveUp, SIGNAL(triggered()), SLOT(moveEntryUp()));
@@ -534,6 +562,7 @@ MainWindow::MainWindow()
     connect(m_ui->actionUserGuide, SIGNAL(triggered()), SLOT(openUserGuide()));
     connect(m_ui->actionOnlineHelp, SIGNAL(triggered()), SLOT(openOnlineHelp()));
     connect(m_ui->actionKeyboardShortcuts, SIGNAL(triggered()), SLOT(openKeyboardShortcuts()));
+    connect(m_ui->actionAllowScreenCapture, &QAction::toggled, this, &MainWindow::setAllowScreenCapture);
 
     connect(osUtils, &OSUtilsBase::statusbarThemeChanged, this, &MainWindow::updateTrayIcon);
 
@@ -646,6 +675,7 @@ MainWindow::MainWindow()
     connect(qApp, SIGNAL(openFile(QString)), this, SLOT(openDatabase(QString)));
     connect(qApp, SIGNAL(quitSignalReceived()), this, SLOT(appExit()), Qt::DirectConnection);
 
+    // Setup the status bar
     statusBar()->setFixedHeight(24);
     m_progressBarLabel = new QLabel(statusBar());
     m_progressBarLabel->setVisible(false);
@@ -658,8 +688,12 @@ MainWindow::MainWindow()
     m_progressBar->setMaximum(100);
     statusBar()->addPermanentWidget(m_progressBar);
     connect(clipboard(), SIGNAL(updateCountdown(int, QString)), this, SLOT(updateProgressBar(int, QString)));
+    m_statusBarLabel = new QLabel(statusBar());
+    m_statusBarLabel->setObjectName("statusBarLabel");
+    statusBar()->addPermanentWidget(m_statusBarLabel);
 
     restoreConfigState();
+    setMenuActionState();
 }
 
 MainWindow::~MainWindow()
@@ -674,13 +708,6 @@ MainWindow::~MainWindow()
  */
 void MainWindow::restoreConfigState()
 {
-    // start minimized if configured
-    if (config()->get(Config::GUI_MinimizeOnStartup).toBool()) {
-        hideWindow();
-    } else {
-        bringToFront();
-    }
-
     if (config()->get(Config::OpenPreviousDatabasesOnStartup).toBool()) {
         const QStringList fileNames = config()->get(Config::LastOpenedDatabases).toStringList();
         for (const QString& filename : fileNames) {
@@ -715,6 +742,43 @@ void MainWindow::appExit()
     close();
 }
 
+/**
+ * Returns if application was built with hardware key support.
+ * Intented to be used by 3rd-party applications using DBus.
+ *
+ * @return True if built with hardware key support, false otherwise
+ */
+bool MainWindow::isHardwareKeySupported()
+{
+#ifdef WITH_XC_YUBIKEY
+    return true;
+#else
+    return false;
+#endif
+}
+
+/**
+ * Refreshes list of hardware keys known.
+ * Triggers the DatabaseOpenWidget to automatically select the key last used for a database if found.
+ * Intented to be used by 3rd-party applications using DBus.
+ *
+ * @return True if any key was found, false otherwise or if application lacks hardware key support
+ */
+bool MainWindow::refreshHardwareKeys()
+{
+#ifdef WITH_XC_YUBIKEY
+    auto yk = YubiKey::instance();
+    // find keys sync to allow returning if any key was found
+    bool found = yk->findValidKeys();
+    // emit signal so DatabaseOpenWidget can select last used key
+    // emit here manually because sync findValidKeys() cannot do that properly
+    emit yk->detectComplete(found);
+    return found;
+#else
+    return false;
+#endif
+}
+
 void MainWindow::updateLastDatabasesMenu()
 {
     m_ui->menuRecentDatabases->clear();
@@ -753,6 +817,38 @@ void MainWindow::updateCopyAttributesMenu()
     }
 }
 
+void MainWindow::updateSetTagsMenu()
+{
+    // Remove all existing actions
+    m_ui->menuTags->clear();
+
+    auto dbWidget = m_ui->tabWidget->currentDatabaseWidget();
+    if (dbWidget) {
+        // Enumerate tags applied to the selected entries
+        QSet<QString> selectedTags;
+        for (auto entry : dbWidget->entryView()->selectedEntries()) {
+            for (auto tag : entry->tagList()) {
+                selectedTags.insert(tag);
+            }
+        }
+
+        // Add known database tags as actions and set checked if
+        // a selected entry has that tag
+        for (auto tag : dbWidget->database()->tagList()) {
+            auto action = m_ui->menuTags->addAction(icons()->icon("tag"), tag);
+            action->setCheckable(true);
+            action->setChecked(selectedTags.contains(tag));
+            m_setTagsMenuActions->addAction(action);
+        }
+    }
+
+    // If no tags exist in the database then show a tip to the user
+    if (m_ui->menuTags->isEmpty()) {
+        auto action = m_ui->menuTags->addAction(tr("No Tags"));
+        action->setEnabled(false);
+    }
+}
+
 void MainWindow::openRecentDatabase(QAction* action)
 {
     openDatabase(action->data().toString());
@@ -781,6 +877,7 @@ void MainWindow::setMenuActionState(DatabaseWidget::Mode mode)
     bool inWelcomeWidget = (currentIndex == WelcomeScreen);
     bool inDatabaseTabWidgetOrWelcomeWidget = inDatabaseTabWidget || inWelcomeWidget;
 
+    m_ui->actionDatabaseClose->setEnabled(true);
     m_ui->actionDatabaseMerge->setEnabled(inDatabaseTabWidget);
     m_ui->actionDatabaseNew->setEnabled(inDatabaseTabWidgetOrWelcomeWidget);
     m_ui->actionDatabaseOpen->setEnabled(inDatabaseTabWidgetOrWelcomeWidget);
@@ -832,8 +929,10 @@ void MainWindow::setMenuActionState(DatabaseWidget::Mode mode)
             m_ui->actionEntryCopyNotes->setEnabled(singleEntrySelected && dbWidget->currentEntryHasNotes());
             m_ui->menuEntryCopyAttribute->setEnabled(singleEntrySelected);
             m_ui->menuEntryTotp->setEnabled(singleEntrySelected);
-            m_ui->actionEntryAutoType->setEnabled(singleEntrySelected);
-            m_ui->actionEntryAutoType->menu()->setEnabled(singleEntrySelected);
+            m_ui->menuTags->setEnabled(entriesSelected);
+            m_ui->actionEntryAutoType->setEnabled(singleEntrySelected && dbWidget->currentEntryHasAutoTypeEnabled());
+            m_ui->actionEntryAutoType->menu()->setEnabled(singleEntrySelected
+                                                          && dbWidget->currentEntryHasAutoTypeEnabled());
             m_ui->actionEntryAutoTypeSequence->setText(
                 singleEntrySelected ? dbWidget->currentSelectedEntry()->effectiveAutoTypeSequence()
                                     : Group::RootAutoTypeSequence);
@@ -849,6 +948,7 @@ void MainWindow::setMenuActionState(DatabaseWidget::Mode mode)
             m_ui->actionEntryOpenUrl->setEnabled(singleEntrySelected && dbWidget->currentEntryHasUrl());
             m_ui->actionEntryTotp->setEnabled(singleEntrySelected && dbWidget->currentEntryHasTotp());
             m_ui->actionEntryCopyTotp->setEnabled(singleEntrySelected && dbWidget->currentEntryHasTotp());
+            m_ui->actionEntryCopyPasswordTotp->setEnabled(singleEntrySelected && dbWidget->currentEntryHasTotp());
             m_ui->actionEntrySetupTotp->setEnabled(singleEntrySelected);
             m_ui->actionEntryTotpQRCode->setEnabled(singleEntrySelected && dbWidget->currentEntryHasTotp());
             m_ui->actionEntryDownloadIcon->setEnabled((entriesSelected && !singleEntrySelected)
@@ -875,6 +975,7 @@ void MainWindow::setMenuActionState(DatabaseWidget::Mode mode)
             m_ui->menuExport->setEnabled(true);
             m_ui->actionExportCsv->setEnabled(true);
             m_ui->actionExportHtml->setEnabled(true);
+            m_ui->actionExportXML->setEnabled(true);
             m_ui->actionDatabaseMerge->setEnabled(m_ui->tabWidget->currentIndex() != -1);
 #ifdef WITH_XC_SSHAGENT
             bool singleEntryHasSshKey =
@@ -935,6 +1036,13 @@ void MainWindow::setMenuActionState(DatabaseWidget::Mode mode)
             // Only disable the action in the database menu so that the
             // menu remains active in the toolbar, if necessary
             m_ui->actionLockDatabase->setEnabled(false);
+            // Never show in these modes
+            m_ui->actionEntryMoveUp->setVisible(false);
+            m_ui->actionEntryMoveDown->setVisible(false);
+            m_ui->actionEntryRestore->setVisible(false);
+            m_ui->actionEntryAddToAgent->setVisible(false);
+            m_ui->actionEntryRemoveFromAgent->setVisible(false);
+            m_ui->actionGroupEmptyRecycleBin->setVisible(false);
 
             m_searchWidgetAction->setEnabled(false);
             break;
@@ -942,7 +1050,6 @@ void MainWindow::setMenuActionState(DatabaseWidget::Mode mode)
         default:
             Q_ASSERT(false);
         }
-        m_ui->actionDatabaseClose->setEnabled(true);
     } else {
         const auto entryActions = m_ui->menuEntries->actions();
         for (auto action : entryActions) {
@@ -965,6 +1072,13 @@ void MainWindow::setMenuActionState(DatabaseWidget::Mode mode)
         m_ui->actionExportCsv->setEnabled(false);
         m_ui->actionExportHtml->setEnabled(false);
         m_ui->actionDatabaseMerge->setEnabled(false);
+        // Hide entry-specific actions
+        m_ui->actionEntryMoveUp->setVisible(false);
+        m_ui->actionEntryMoveDown->setVisible(false);
+        m_ui->actionEntryRestore->setVisible(false);
+        m_ui->actionEntryAddToAgent->setVisible(false);
+        m_ui->actionEntryRemoveFromAgent->setVisible(false);
+        m_ui->actionGroupEmptyRecycleBin->setVisible(false);
 
         m_searchWidgetAction->setEnabled(false);
     }
@@ -1248,6 +1362,25 @@ void MainWindow::databaseTabChanged(int tabIndex)
     }
 
     m_actionMultiplexer.setCurrentObject(m_ui->tabWidget->currentDatabaseWidget());
+    updateEntryCountLabel();
+}
+
+void MainWindow::showEvent(QShowEvent* event)
+{
+    Q_UNUSED(event)
+#ifdef Q_OS_WIN
+    // Qt Hack - Prevent white flicker when showing window
+    QTimer::singleShot(50, this, [=] { setProperty("windowOpacity", 1.0); });
+#endif
+}
+
+void MainWindow::hideEvent(QHideEvent* event)
+{
+    Q_UNUSED(event)
+#ifdef Q_OS_WIN
+    // Qt Hack - Prevent white flicker when showing window
+    setProperty("windowOpacity", 0.0);
+#endif
 }
 
 void MainWindow::closeEvent(QCloseEvent* event)
@@ -1288,7 +1421,7 @@ void MainWindow::changeEvent(QEvent* event)
         }
 
         if (config()->get(Config::Security_LockDatabaseMinimize).toBool()) {
-            m_ui->tabWidget->lockDatabases();
+            m_ui->tabWidget->lockDatabasesDelayed();
         }
     } else {
         QMainWindow::changeEvent(event);
@@ -1307,7 +1440,7 @@ void MainWindow::keyPressEvent(QKeyEvent* event)
             } else if (event->key() == Qt::Key_F2) {
                 dbWidget->focusOnEntries(true);
                 return;
-            } else if (event->key() == Qt::Key_F3) {
+            } else if (event->key() == Qt::Key_F3 || event->key() == Qt::Key_F6) {
                 focusSearchWidget();
                 return;
             } else if (event->key() == Qt::Key_Escape && dbWidget->isSearchActive()) {
@@ -1388,7 +1521,7 @@ bool MainWindow::saveLastDatabases()
         QStringList openDatabases;
         for (int i = 0; i < m_ui->tabWidget->count(); ++i) {
             auto dbWidget = m_ui->tabWidget->databaseWidgetFromIndex(i);
-            openDatabases.append(dbWidget->database()->filePath());
+            openDatabases.append(QDir::toNativeSeparators(dbWidget->database()->filePath()));
         }
 
         config()->set(Config::LastOpenedDatabases, openDatabases);
@@ -1428,12 +1561,8 @@ void MainWindow::updateTrayIcon()
             connect(actionToggle, SIGNAL(triggered()), SLOT(toggleWindow()));
         }
 
-        if (m_ui->tabWidget->hasLockableDatabases()) {
-            m_trayIcon->setIcon(icons()->trayIconUnlocked());
-        } else {
-            m_trayIcon->setIcon(icons()->trayIconLocked());
-        }
-
+        bool showUnlocked = m_ui->tabWidget->hasLockableDatabases();
+        m_trayIcon->setIcon(icons()->trayIcon(showUnlocked));
         m_trayIcon->setToolTip(windowTitle().replace("[*]", isWindowModified() ? "*" : ""));
         m_trayIcon->show();
 
@@ -1469,6 +1598,17 @@ void MainWindow::updateProgressBar(int percentage, QString message)
     }
 }
 
+void MainWindow::updateEntryCountLabel()
+{
+    auto dbWidget = m_ui->tabWidget->currentDatabaseWidget();
+    if (dbWidget && dbWidget->currentMode() == DatabaseWidget::Mode::ViewMode) {
+        int numEntries = dbWidget->entryView()->model()->rowCount();
+        m_statusBarLabel->setText(tr("%1 Entry(s)", "", numEntries).arg(numEntries));
+    } else {
+        m_statusBarLabel->setText("");
+    }
+}
+
 void MainWindow::obtainContextFocusLock()
 {
     m_contextMenuFocusLock = true;
@@ -1529,7 +1669,12 @@ void MainWindow::applySettingsChanges()
     }
 
     m_ui->toolBar->setHidden(config()->get(Config::GUI_HideToolbar).toBool());
-    m_ui->toolBar->setMovable(config()->get(Config::GUI_MovableToolbar).toBool());
+    auto movable = config()->get(Config::GUI_MovableToolbar).toBool();
+    m_ui->toolBar->setMovable(movable);
+    if (!movable) {
+        // Move the toolbar back to the top of the main window
+        addToolBar(Qt::TopToolBarArea, m_ui->toolBar);
+    }
 
     bool isOk = false;
     const auto toolButtonStyle =
@@ -1541,11 +1686,29 @@ void MainWindow::applySettingsChanges()
     updateTrayIcon();
 }
 
-void MainWindow::focusWindowChanged(QWindow* focusWindow)
+void MainWindow::setAllowScreenCapture(bool state)
 {
-    if (focusWindow != windowHandle()) {
+    m_allowScreenCapture = state;
+    for (auto window : qApp->topLevelWindows()) {
+        if (window->isVisible()) {
+            osUtils->setPreventScreenCapture(window, !m_allowScreenCapture);
+        }
+    }
+    m_ui->actionAllowScreenCapture->blockSignals(true);
+    m_ui->actionAllowScreenCapture->setChecked(m_allowScreenCapture);
+    m_ui->actionAllowScreenCapture->blockSignals(false);
+}
+
+void MainWindow::focusWindowChanged(QWindow* window)
+{
+    if (window != windowHandle()) {
         m_lastFocusOutTime = Clock::currentMilliSecondsSinceEpoch();
     }
+
+    if (!osUtils->setPreventScreenCapture(window, !m_allowScreenCapture) && !m_allowScreenCapture) {
+        displayGlobalMessage(QObject::tr("Warning: Failed to block screenshot capture on a top-level window."),
+                             MessageWidget::Error);
+    }
 }
 
 void MainWindow::trayIconTriggered(QSystemTrayIcon::ActivationReason reason)
@@ -1636,7 +1799,7 @@ void MainWindow::hideWindow()
     }
 
     if (config()->get(Config::Security_LockDatabaseMinimize).toBool()) {
-        m_ui->tabWidget->lockDatabases();
+        m_ui->tabWidget->lockDatabasesDelayed();
     }
 }
 
