@@ -20,10 +20,9 @@
 #define KEEPASSX_CSVPARSERMODEL_H
 
 #include <QAbstractTableModel>
-#include <QMap>
 
-#include "core/CsvParser.h"
 #include "core/Group.h"
+#include "format/CsvParser.h"
 
 class CsvParserModel : public QAbstractTableModel, public CsvParser
 {
