@@ -25,18 +25,18 @@
 #ifdef WITH_XC_BROWSER
 #include "DatabaseSettingsWidgetBrowser.h"
 #endif
-#if defined(WITH_XC_KEESHARE)
+#include "DatabaseSettingsWidgetMaintenance.h"
+#ifdef WITH_XC_KEESHARE
 #include "keeshare/DatabaseSettingsPageKeeShare.h"
 #endif
-#if defined(WITH_XC_FDOSECRETS)
+#ifdef WITH_XC_FDOSECRETS
 #include "fdosecrets/DatabaseSettingsPageFdoSecrets.h"
 #endif
 
 #include "core/Config.h"
 #include "core/Database.h"
 #include "core/Global.h"
-#include "core/Resources.h"
-#include "touchid/TouchID.h"
+#include "gui/Icons.h"
 
 #include <QScrollArea>
 
@@ -72,14 +72,15 @@ DatabaseSettingsDialog::DatabaseSettingsDialog(QWidget* parent)
 #ifdef WITH_XC_BROWSER
     , m_browserWidget(new DatabaseSettingsWidgetBrowser(this))
 #endif
+    , m_maintenanceWidget(new DatabaseSettingsWidgetMaintenance(this))
 {
     m_ui->setupUi(this);
 
     connect(m_ui->buttonBox, SIGNAL(accepted()), SLOT(save()));
     connect(m_ui->buttonBox, SIGNAL(rejected()), SLOT(reject()));
 
-    m_ui->categoryList->addCategory(tr("General"), Resources::instance()->icon("preferences-other"));
-    m_ui->categoryList->addCategory(tr("Security"), Resources::instance()->icon("security-high"));
+    m_ui->categoryList->addCategory(tr("General"), icons()->icon("preferences-other"));
+    m_ui->categoryList->addCategory(tr("Security"), icons()->icon("security-high"));
     m_ui->stackedWidget->addWidget(m_generalWidget);
 
     m_ui->stackedWidget->addWidget(m_securityTabWidget);
@@ -111,10 +112,13 @@ DatabaseSettingsDialog::DatabaseSettingsDialog(QWidget* parent)
     connect(m_ui->advancedSettingsToggle, SIGNAL(toggled(bool)), SLOT(toggleAdvancedMode(bool)));
 
 #ifdef WITH_XC_BROWSER
-    m_ui->categoryList->addCategory(tr("Browser Integration"), Resources::instance()->icon("internet-web-browser"));
+    m_ui->categoryList->addCategory(tr("Browser Integration"), icons()->icon("internet-web-browser"));
     m_ui->stackedWidget->addWidget(m_browserWidget);
 #endif
 
+    m_ui->categoryList->addCategory(tr("Maintenance"), icons()->icon("hammer-wrench"));
+    m_ui->stackedWidget->addWidget(m_maintenanceWidget);
+
     pageChanged();
 }
 
@@ -131,6 +135,7 @@ void DatabaseSettingsDialog::load(const QSharedPointer<Database>& db)
 #ifdef WITH_XC_BROWSER
     m_browserWidget->load(db);
 #endif
+    m_maintenanceWidget->load(db);
     for (const ExtraPage& page : asConst(m_extraPages)) {
         page.loadSettings(db);
     }
@@ -176,10 +181,6 @@ void DatabaseSettingsDialog::save()
         extraPage.saveSettings();
     }
 
-#ifdef WITH_XC_TOUCHID
-    TouchID::getInstance().reset(m_db ? m_db->filePath() : "");
-#endif
-
     emit editFinished(true);
 }
 
