@@ -17,11 +17,9 @@
 
 #include "EntryAttachmentsModel.h"
 
-#include "core/Entry.h"
+#include "core/EntryAttachments.h"
 #include "core/Tools.h"
 
-#include <algorithm>
-
 EntryAttachmentsModel::EntryAttachmentsModel(QObject* parent)
     : QAbstractListModel(parent)
     , m_entryAttachments(nullptr)
@@ -101,6 +99,28 @@ QVariant EntryAttachmentsModel::data(const QModelIndex& index, int role) const
     return QVariant();
 }
 
+bool EntryAttachmentsModel::setData(const QModelIndex& index, const QVariant& value, int role)
+{
+    if (!m_readOnly && index.column() == Columns::NameColumn) {
+        const QString key = value.toString().trimmed();
+        if (key.isEmpty() || m_entryAttachments->hasKey(key)) {
+            return false;
+        }
+        m_entryAttachments->rename(keyByIndex(index), key);
+        return true;
+    }
+    return QAbstractListModel::setData(index, value, role);
+}
+
+Qt::ItemFlags EntryAttachmentsModel::flags(const QModelIndex& index) const
+{
+    Qt::ItemFlags flags = QAbstractListModel::flags(index);
+    if (!m_readOnly && index.column() == Columns::NameColumn) {
+        flags = flags | Qt::ItemIsEditable;
+    }
+    return flags;
+}
+
 QString EntryAttachmentsModel::keyByIndex(const QModelIndex& index) const
 {
     if (!index.isValid()) {
@@ -150,3 +170,8 @@ void EntryAttachmentsModel::reset()
 {
     endResetModel();
 }
+
+void EntryAttachmentsModel::setReadOnly(bool readOnly)
+{
+    m_readOnly = readOnly;
+}
