@@ -1,6 +1,6 @@
 /*
+ *  Copyright (C) 2023 KeePassXC Team <team@keepassxc.org>
  *  Copyright (C) 2012 Felix Geyer <debfx@fobos.de>
- *  Copyright (C) 2019 KeePassXC Team <team@keepassxc.org>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,17 +18,16 @@
 
 #include "EntryURLModel.h"
 
-#include "core/Entry.h"
-#include "core/Resources.h"
+#include "browser/BrowserService.h"
+#include "core/EntryAttributes.h"
 #include "core/Tools.h"
+#include "gui/Icons.h"
 #include "gui/styles/StateColorPalette.h"
 
-#include <algorithm>
-
 EntryURLModel::EntryURLModel(QObject* parent)
     : QStandardItemModel(parent)
     , m_entryAttributes(nullptr)
-    , m_errorIcon(resources()->icon("dialog-error"))
+    , m_errorIcon(icons()->icon("dialog-error"))
 {
 }
 
@@ -70,13 +69,21 @@ QVariant EntryURLModel::data(const QModelIndex& index, int role) const
     const auto value = m_entryAttributes->value(key);
     const auto urlValid = Tools::checkUrlValid(value);
 
-    if (role == Qt::BackgroundRole && !urlValid) {
+    // Check for duplicate URLs in the attribute list. Excludes the current key/value from the comparison.
+    auto customAttributeKeys = m_entryAttributes->customKeys().filter(BrowserService::ADDITIONAL_URL);
+    customAttributeKeys.removeOne(key);
+
+    const auto duplicateUrl = m_entryAttributes->values(customAttributeKeys).contains(value)
+                              || browserService()->isUrlIdentical(value, m_entryUrl);
+    if (role == Qt::BackgroundRole && (!urlValid || duplicateUrl)) {
         StateColorPalette statePalette;
         return statePalette.color(StateColorPalette::ColorRole::Error);
-    } else if (role == Qt::DecorationRole && !urlValid) {
+    } else if (role == Qt::DecorationRole && (!urlValid || duplicateUrl)) {
         return m_errorIcon;
     } else if (role == Qt::DisplayRole || role == Qt::EditRole) {
         return value;
+    } else if (role == Qt::ToolTipRole && duplicateUrl) {
+        return tr("Duplicate URL");
     } else if (role == Qt::ToolTipRole && !urlValid) {
         return tr("Invalid URL");
     }
@@ -84,17 +91,21 @@ QVariant EntryURLModel::data(const QModelIndex& index, int role) const
     return {};
 }
 
+void EntryURLModel::setEntryUrl(const QString& entryUrl)
+{
+    m_entryUrl = entryUrl;
+}
+
 bool EntryURLModel::setData(const QModelIndex& index, const QVariant& value, int role)
 {
     if (!index.isValid() || role != Qt::EditRole || value.type() != QVariant::String || value.toString().isEmpty()) {
         return false;
     }
 
-    const int row = index.row();
-    const QString key = m_urls.at(row).first;
-    const QString oldValue = m_urls.at(row).second;
+    const auto row = index.row();
+    const auto key = m_urls.at(row).first;
 
-    if (EntryAttributes::isDefaultAttribute(key) || m_entryAttributes->containsValue(value.toString())) {
+    if (EntryAttributes::isDefaultAttribute(key)) {
         return false;
     }
 
@@ -115,7 +126,7 @@ QModelIndex EntryURLModel::indexByKey(const QString& key) const
     }
 
     if (row == -1) {
-        return QModelIndex();
+        return {};
     } else {
         return index(row, 0);
     }
@@ -124,7 +135,7 @@ QModelIndex EntryURLModel::indexByKey(const QString& key) const
 QString EntryURLModel::keyByIndex(const QModelIndex& index) const
 {
     if (!index.isValid()) {
-        return QString();
+        return {};
     } else {
         return m_urls.at(index.row()).first;
     }
@@ -135,9 +146,9 @@ void EntryURLModel::updateAttributes()
     clear();
     m_urls.clear();
 
-    const QList<QString> attributesKeyList = m_entryAttributes->keys();
-    for (const QString& key : attributesKeyList) {
-        if (!EntryAttributes::isDefaultAttribute(key) && key.contains("KP2A_URL")) {
+    const auto attributesKeyList = m_entryAttributes->keys();
+    for (const auto& key : attributesKeyList) {
+        if (!EntryAttributes::isDefaultAttribute(key) && key.contains(BrowserService::ADDITIONAL_URL)) {
             const auto value = m_entryAttributes->value(key);
             m_urls.append(qMakePair(key, value));
 
