@@ -19,8 +19,9 @@
 #ifndef KEEPASSXC_MACUTILS_H
 #define KEEPASSXC_MACUTILS_H
 
-#include "gui/osutils/OSUtilsBase.h"
 #include "AppKit.h"
+#include "gui/osutils/OSUtilsBase.h"
+#include <Carbon/Carbon.h>
 
 #include <QColor>
 #include <QPointer>
@@ -50,6 +51,20 @@ public:
     bool enableScreenRecording();
     void toggleForegroundApp(bool foreground);
 
+    void registerNativeEventFilter() override;
+
+    bool registerGlobalShortcut(const QString& name,
+                                Qt::Key key,
+                                Qt::KeyboardModifiers modifiers,
+                                QString* error = nullptr) override;
+    bool unregisterGlobalShortcut(const QString& name) override;
+
+    uint16 qtToNativeKeyCode(Qt::Key key);
+    CGEventFlags qtToNativeModifiers(Qt::KeyboardModifiers modifiers, bool native);
+
+    bool canPreventScreenCapture() const override;
+    bool setPreventScreenCapture(QWindow* window, bool prevent) const override;
+
 signals:
     void lockDatabases();
 
@@ -59,10 +74,22 @@ protected:
 
 private:
     QString getLaunchAgentFilename() const;
+    static OSStatus hotkeyHandler(EventHandlerCallRef nextHandler, EventRef theEvent, void* userData);
 
     QScopedPointer<AppKit> m_appkit;
     static QPointer<MacUtils> m_instance;
 
+    struct globalShortcut
+    {
+        EventHotKeyRef hotkeyRef;
+        EventHotKeyID hotkeyId;
+        uint16 nativeKeyCode;
+        CGEventFlags nativeModifiers;
+    };
+
+    int m_nextShortcutId = 1;
+    QHash<QString, QSharedPointer<globalShortcut>> m_globalShortcuts;
+
     Q_DISABLE_COPY(MacUtils)
 };
 
