@@ -19,15 +19,15 @@
 #include "ui_ReportsWidgetHealthcheck.h"
 
 #include "core/AsyncTask.h"
-#include "core/Database.h"
-#include "core/Global.h"
 #include "core/Group.h"
+#include "core/Metadata.h"
 #include "core/PasswordHealth.h"
-#include "core/Resources.h"
+#include "gui/GuiTools.h"
+#include "gui/Icons.h"
 #include "gui/styles/StateColorPalette.h"
 
 #include <QMenu>
-#include <QSharedPointer>
+#include <QShortcut>
 #include <QSortFilterProxyModel>
 #include <QStandardItemModel>
 
@@ -38,17 +38,16 @@ namespace
     public:
         struct Item
         {
-            QPointer<const Group> group;
-            QPointer<const Entry> entry;
+            QPointer<Group> group;
+            QPointer<Entry> entry;
             QSharedPointer<PasswordHealth> health;
-            bool knownBad = false;
+            bool exclude = false;
 
-            Item(const Group* g, const Entry* e, QSharedPointer<PasswordHealth> h)
+            Item(Group* g, Entry* e, QSharedPointer<PasswordHealth> h)
                 : group(g)
                 , entry(e)
                 , health(h)
-                , knownBad(e->customData()->contains(PasswordHealth::OPTION_KNOWN_BAD)
-                           && e->customData()->value(PasswordHealth::OPTION_KNOWN_BAD) == TRUE_STR)
+                , exclude(e->excludeFromReports())
             {
             }
 
@@ -103,13 +102,13 @@ Health::Health(QSharedPointer<Database> db)
     : m_db(db)
     , m_checker(db)
 {
-    for (const auto* group : db->rootGroup()->groupsRecursive(true)) {
+    for (auto group : db->rootGroup()->groupsRecursive(true)) {
         // Skip recycle bin
         if (group->isRecycled()) {
             continue;
         }
 
-        for (const auto* entry : group->entries()) {
+        for (auto entry : group->entries()) {
             if (entry->isRecycled()) {
                 continue;
             }
@@ -121,7 +120,7 @@ Health::Health(QSharedPointer<Database> db)
 
             // Evaluate this entry
             const auto item = QSharedPointer<Item>(new Item(group, entry, m_checker.evaluate(entry)));
-            if (item->knownBad) {
+            if (item->exclude) {
                 m_anyKnownBad = true;
             }
 
@@ -140,7 +139,6 @@ Health::Health(QSharedPointer<Database> db)
 ReportsWidgetHealthcheck::ReportsWidgetHealthcheck(QWidget* parent)
     : QWidget(parent)
     , m_ui(new Ui::ReportsWidgetHealthcheck())
-    , m_errorIcon(Resources::instance()->icon("dialog-error"))
     , m_referencesModel(new QStandardItemModel(this))
     , m_modelProxy(new ReportSortProxyModel(this))
 {
@@ -149,15 +147,15 @@ ReportsWidgetHealthcheck::ReportsWidgetHealthcheck(QWidget* parent)
     m_modelProxy->setSourceModel(m_referencesModel.data());
     m_modelProxy->setSortLocaleAware(true);
     m_ui->healthcheckTableView->setModel(m_modelProxy.data());
-    m_ui->healthcheckTableView->setSelectionMode(QAbstractItemView::NoSelection);
     m_ui->healthcheckTableView->horizontalHeader()->setSectionResizeMode(QHeaderView::Interactive);
     m_ui->healthcheckTableView->verticalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
-    m_ui->healthcheckTableView->setSortingEnabled(true);
-    m_ui->healthcheckTableView->setWordWrap(true);
 
     connect(m_ui->healthcheckTableView, SIGNAL(customContextMenuRequested(QPoint)), SLOT(customMenuRequested(QPoint)));
     connect(m_ui->healthcheckTableView, SIGNAL(doubleClicked(QModelIndex)), SLOT(emitEntryActivated(QModelIndex)));
     connect(m_ui->showKnownBadCheckBox, SIGNAL(stateChanged(int)), this, SLOT(calculateHealth()));
+    connect(m_ui->excludeExpired, SIGNAL(stateChanged(int)), this, SLOT(calculateHealth()));
+
+    new QShortcut(Qt::Key_Delete, this, SLOT(deleteSelectedEntries()));
 }
 
 ReportsWidgetHealthcheck::~ReportsWidgetHealthcheck()
@@ -165,8 +163,8 @@ ReportsWidgetHealthcheck::~ReportsWidgetHealthcheck()
 }
 
 void ReportsWidgetHealthcheck::addHealthRow(QSharedPointer<PasswordHealth> health,
-                                            const Group* group,
-                                            const Entry* entry,
+                                            Group* group,
+                                            Entry* entry,
                                             bool knownBad)
 {
     QString descr, tip;
@@ -205,8 +203,8 @@ void ReportsWidgetHealthcheck::addHealthRow(QSharedPointer<PasswordHealth> healt
 
     auto row = QList<QStandardItem*>();
     row << new QStandardItem(descr);
-    row << new QStandardItem(entry->iconPixmap(), title);
-    row << new QStandardItem(group->iconPixmap(), group->hierarchy().join("/"));
+    row << new QStandardItem(Icons::entryIconPixmap(entry), title);
+    row << new QStandardItem(Icons::groupIconPixmap(group), group->hierarchy().join("/"));
     row << new QStandardItem(QString::number(health->score()));
     row << new QStandardItem(health->scoreReason());
 
@@ -237,7 +235,7 @@ void ReportsWidgetHealthcheck::loadSettings(QSharedPointer<Database> db)
     m_rowToEntry.clear();
 
     auto row = QList<QStandardItem*>();
-    row << new QStandardItem(tr("Please wait, health data is being calculated..."));
+    row << new QStandardItem(tr("Please wait, health data is being calculated…"));
     m_referencesModel->appendRow(row);
 }
 
@@ -260,18 +258,19 @@ void ReportsWidgetHealthcheck::calculateHealth()
     const QScopedPointer<Health> health(AsyncTask::runAndWaitForFuture([this] { return new Health(m_db); }));
 
     // Display entries that are marked as "known bad"?
-    const auto showKnownBad = m_ui->showKnownBadCheckBox->isChecked();
+    const auto showExcluded = m_ui->showKnownBadCheckBox->isChecked();
 
     // Display the entries
     m_rowToEntry.clear();
     for (const auto& item : health->items()) {
-        if (item->knownBad && !showKnownBad) {
+        auto excluded = item->exclude || (item->entry->isExpired() && m_ui->excludeExpired->isChecked());
+        if (excluded && !showExcluded) {
             // Exclude this entry from the report
             continue;
         }
 
         // Show the entry in the report
-        addHealthRow(item->health, item->group, item->entry, item->knownBad);
+        addHealthRow(item->health, item->group, item->entry, item->exclude);
     }
 
     // Set the table header
@@ -312,57 +311,82 @@ void ReportsWidgetHealthcheck::emitEntryActivated(const QModelIndex& index)
 
 void ReportsWidgetHealthcheck::customMenuRequested(QPoint pos)
 {
-
-    // Find which entry has been clicked
-    const auto index = m_ui->healthcheckTableView->indexAt(pos);
-    if (!index.isValid()) {
-        return;
-    }
-    auto mappedIndex = m_modelProxy->mapToSource(index);
-    m_contextmenuEntry = const_cast<Entry*>(m_rowToEntry[mappedIndex.row()].second);
-    if (!m_contextmenuEntry) {
+    auto selected = m_ui->healthcheckTableView->selectionModel()->selectedRows();
+    if (selected.isEmpty()) {
         return;
     }
 
     // Create the context menu
     const auto menu = new QMenu(this);
 
-    // Create the "edit entry" menu item
-    const auto edit = new QAction(Resources::instance()->icon("entry-edit"), tr("Edit Entry..."), this);
-    menu->addAction(edit);
-    connect(edit, SIGNAL(triggered()), SLOT(editFromContextmenu()));
+    // Create the "edit entry" menu item (only if 1 row is selected)
+    if (selected.size() == 1) {
+        const auto edit = new QAction(icons()->icon("entry-edit"), tr("Edit Entry…"), this);
+        menu->addAction(edit);
+        connect(edit, &QAction::triggered, edit, [this, selected] {
+            auto row = m_modelProxy->mapToSource(selected[0]).row();
+            auto entry = m_rowToEntry[row].second;
+            emit entryActivated(entry);
+        });
+    }
+
+    // Create the "delete entry" menu item
+    const auto delEntry = new QAction(icons()->icon("entry-delete"), tr("Delete Entry(s)…", "", selected.size()), this);
+    menu->addAction(delEntry);
+    connect(delEntry, &QAction::triggered, this, &ReportsWidgetHealthcheck::deleteSelectedEntries);
 
     // Create the "exclude from reports" menu item
-    const auto knownbad = new QAction(Resources::instance()->icon("reports-exclude"), tr("Exclude from reports"), this);
-    knownbad->setCheckable(true);
-    knownbad->setChecked(m_contextmenuEntry->customData()->contains(PasswordHealth::OPTION_KNOWN_BAD)
-                         && m_contextmenuEntry->customData()->value(PasswordHealth::OPTION_KNOWN_BAD) == TRUE_STR);
-    menu->addAction(knownbad);
-    connect(knownbad, SIGNAL(toggled(bool)), SLOT(toggleKnownBad(bool)));
+    const auto exclude = new QAction(icons()->icon("reports-exclude"), tr("Exclude from reports"), this);
+
+    bool isExcluded = false;
+    for (auto index : selected) {
+        auto row = m_modelProxy->mapToSource(index).row();
+        auto entry = m_rowToEntry[row].second;
+        if (entry && entry->excludeFromReports()) {
+            // If at least one entry is excluded switch to inclusion
+            isExcluded = true;
+            break;
+        }
+    }
+    exclude->setCheckable(true);
+    exclude->setChecked(isExcluded);
+
+    menu->addAction(exclude);
+    connect(exclude, &QAction::toggled, exclude, [this, selected](bool state) {
+        for (auto index : selected) {
+            auto row = m_modelProxy->mapToSource(index).row();
+            auto entry = m_rowToEntry[row].second;
+            if (entry) {
+                entry->setExcludeFromReports(state);
+            }
+        }
+        calculateHealth();
+    });
 
     // Show the context menu
     menu->popup(m_ui->healthcheckTableView->viewport()->mapToGlobal(pos));
 }
 
-void ReportsWidgetHealthcheck::editFromContextmenu()
-{
-    if (m_contextmenuEntry) {
-        emit entryActivated(m_contextmenuEntry);
-    }
-}
-
-void ReportsWidgetHealthcheck::toggleKnownBad(bool isKnownBad)
-{
-    if (!m_contextmenuEntry) {
-        return;
-    }
-
-    m_contextmenuEntry->customData()->set(PasswordHealth::OPTION_KNOWN_BAD, isKnownBad ? TRUE_STR : FALSE_STR);
-
-    calculateHealth();
-}
-
 void ReportsWidgetHealthcheck::saveSettings()
 {
     // nothing to do - the tab is passive
 }
+
+void ReportsWidgetHealthcheck::deleteSelectedEntries()
+{
+    QList<Entry*> selectedEntries;
+    for (auto index : m_ui->healthcheckTableView->selectionModel()->selectedRows()) {
+        auto row = m_modelProxy->mapToSource(index).row();
+        auto entry = m_rowToEntry[row].second;
+        if (entry) {
+            selectedEntries << entry;
+        }
+    }
+
+    bool permanent = !m_db->metadata()->recycleBinEnabled();
+    if (GuiTools::confirmDeleteEntries(this, selectedEntries, permanent)) {
+        GuiTools::deleteEntriesResolveReferences(this, selectedEntries, permanent);
+    }
+
+    calculateHealth();
+}
