@@ -16,15 +16,14 @@
  */
 
 #include "ShareObserver.h"
-#include "core/Config.h"
-#include "core/Database.h"
 #include "core/FileWatcher.h"
-#include "core/Global.h"
 #include "core/Group.h"
 #include "keeshare/KeeShare.h"
 #include "keeshare/ShareExport.h"
 #include "keeshare/ShareImport.h"
 
+#include <QDir>
+
 namespace
 {
     QString resolvePath(const QString& path, QSharedPointer<Database> database)
@@ -41,14 +40,14 @@ ShareObserver::ShareObserver(QSharedPointer<Database> db, QObject* parent)
     : QObject(parent)
     , m_db(std::move(db))
 {
-    connect(KeeShare::instance(), SIGNAL(activeChanged()), SLOT(handleDatabaseChanged()));
+    connect(KeeShare::instance(), &KeeShare::activeChanged, this, &ShareObserver::handleDatabaseChanged);
 
-    connect(m_db.data(), SIGNAL(groupDataChanged(Group*)), SLOT(handleDatabaseChanged()));
-    connect(m_db.data(), SIGNAL(groupAdded()), SLOT(handleDatabaseChanged()));
-    connect(m_db.data(), SIGNAL(groupRemoved()), SLOT(handleDatabaseChanged()));
+    connect(m_db.data(), &Database::groupDataChanged, this, &ShareObserver::handleDatabaseChanged);
+    connect(m_db.data(), &Database::groupAdded, this, &ShareObserver::handleDatabaseChanged);
+    connect(m_db.data(), &Database::groupRemoved, this, &ShareObserver::handleDatabaseChanged);
 
-    connect(m_db.data(), SIGNAL(databaseModified()), SLOT(handleDatabaseChanged()));
-    connect(m_db.data(), SIGNAL(databaseSaved()), SLOT(handleDatabaseSaved()));
+    connect(m_db.data(), &Database::modified, this, &ShareObserver::handleDatabaseChanged);
+    connect(m_db.data(), &Database::databaseSaved, this, &ShareObserver::handleDatabaseSaved);
 
     handleDatabaseChanged();
 }
@@ -279,13 +278,14 @@ QList<ShareObserver::Result> ShareObserver::exportShares()
     }
 
     for (auto it = references.cbegin(); it != references.cend(); ++it) {
-        const auto& reference = it.value().first();
+        auto reference = it.value().first();
         const QString resolvedPath = resolvePath(reference.config.path, m_db);
         auto watcher = m_fileWatchers.value(resolvedPath);
         if (watcher) {
             watcher->stop();
         }
 
+        // TODO: save new path into group settings if not saving to signed container anymore
         results << ShareExport::intoContainer(resolvedPath, reference.config, reference.group);
 
         if (watcher) {
