@@ -17,11 +17,13 @@
  */
 
 #include "TestTotp.h"
-#include "TestGlobal.h"
 
+#include "core/Entry.h"
 #include "crypto/Crypto.h"
 #include "totp/totp.h"
 
+#include <QTest>
+
 QTEST_GUILESS_MAIN(TestTotp)
 
 void TestTotp::initTestCase()
@@ -57,6 +59,12 @@ void TestTotp::testParseSecret()
     QCOMPARE(settings->step, 30u);
     QCOMPARE(settings->algorithm, Totp::Algorithm::Sha512);
 
+    // Max TOTP step of 24-hours
+    secret.replace("period=30", "period=90000");
+    settings = Totp::parseSettings(secret);
+    QVERIFY(!settings.isNull());
+    QCOMPARE(settings->step, 86400u);
+
     // KeeOTP Parsing
     secret = "key=HXDMVJECJJWSRBY%3d&step=25&size=8&otpHashMode=Sha256";
     settings = Totp::parseSettings(secret);
