@@ -91,6 +91,7 @@ void TestGui::initTestCase()
 
     m_mainWindow.reset(new MainWindow());
     m_tabWidget = m_mainWindow->findChild<DatabaseTabWidget*>("tabWidget");
+    m_statusBarLabel = m_mainWindow->findChild<QLabel*>("statusBarLabel");
     m_mainWindow->show();
     m_mainWindow->resize(1024, 768);
 }
@@ -286,6 +287,10 @@ void TestGui::testCreateDatabase()
 
     triggerAction("actionDatabaseNew");
 
+    QCOMPARE(m_tabWidget->count(), 2);
+
+    checkStatusBarText("0 Ent");
+
     // there is a new empty db
     m_db = m_tabWidget->currentDatabaseWidget()->database();
     QCOMPARE(m_db->rootGroup()->children().size(), 0);
@@ -306,6 +311,15 @@ void TestGui::testCreateDatabase()
     compositeKey->addKey(fileKey);
     QCOMPARE(m_db->key()->rawKey(), compositeKey->rawKey());
 
+    checkStatusBarText("0 Ent");
+
+    // Test the switching to other DB tab
+    m_tabWidget->setCurrentIndex(0);
+    checkStatusBarText("1 Ent");
+
+    m_tabWidget->setCurrentIndex(1);
+    checkStatusBarText("0 Ent");
+
     // close the new database
     MessageBox::setNextAnswer(MessageBox::No);
     triggerAction("actionDatabaseClose");
@@ -592,6 +606,9 @@ void TestGui::testAddEntry()
     auto* toolBar = m_mainWindow->findChild<QToolBar*>("toolBar");
     auto* entryView = m_dbWidget->findChild<EntryView*>("entryView");
 
+    // Given the status bar label with initial number of entries.
+    checkStatusBarText("1 Ent");
+
     // Find the new entry action
     auto* entryNewAction = m_mainWindow->findChild<QAction*>("actionEntryNew");
     QVERIFY(entryNewAction->isEnabled());
@@ -626,6 +643,9 @@ void TestGui::testAddEntry()
 
     m_db->updateCommonUsernames();
 
+    // Then the status bar label should be updated with incremented number of entries.
+    checkStatusBarText("2 Ent");
+
     // Add entry "something 2"
     QTest::mouseClick(entryNewWidget, Qt::LeftButton);
     QTest::keyClicks(titleEdit, "something 2");
@@ -653,7 +673,7 @@ void TestGui::testAddEntry()
 
     QApplication::processEvents();
 
-    // Confirm entry count
+    // Confirm no changed entry count
     QTRY_COMPARE(entryView->model()->rowCount(), 3);
 }
 
@@ -895,12 +915,27 @@ void TestGui::testTotp()
     auto* editEntryWidgetButtonBox = editEntryWidget->findChild<QDialogButtonBox*>("buttonBox");
     QTest::mouseClick(editEntryWidgetButtonBox->button(QDialogButtonBox::Ok), Qt::LeftButton);
 
+    // Test the TOTP value
     triggerAction("actionEntryTotp");
 
     auto* totpDialog = m_dbWidget->findChild<TotpDialog*>("TotpDialog");
     auto* totpLabel = totpDialog->findChild<QLabel*>("totpLabel");
 
     QCOMPARE(totpLabel->text().replace(" ", ""), entry->totp());
+    QTest::keyClick(totpDialog, Qt::Key_Escape);
+
+    // Test the QR code
+    triggerAction("actionEntryTotpQRCode");
+    auto* qrCodeDialog = m_mainWindow->findChild<QDialog*>("entryQrCodeWidget");
+    QVERIFY(qrCodeDialog);
+    QVERIFY(qrCodeDialog->isVisible());
+    auto* qrCodeWidget = qrCodeDialog->findChild<QWidget*>("squareSvgWidget");
+    QVERIFY2(qrCodeWidget->geometry().width() == qrCodeWidget->geometry().height(), "Initial QR code is not square");
+
+    // Test the QR code window resizing, make the dialog bigger.
+    qrCodeDialog->setFixedSize(800, 600);
+    QVERIFY2(qrCodeWidget->geometry().width() == qrCodeWidget->geometry().height(), "Resized QR code is not square");
+    QTest::keyClick(qrCodeDialog, Qt::Key_Escape);
 }
 
 void TestGui::testSearch()
@@ -935,6 +970,10 @@ void TestGui::testSearch()
     QApplication::processEvents();
     helpButton->trigger();
     QTRY_VERIFY(!helpPanel->isVisible());
+
+    // Need to re-activate the window after the help test
+    m_mainWindow->activateWindow();
+
     // Search for "ZZZ"
     QTest::keyClicks(searchTextEdit, "ZZZ");
     QTRY_COMPARE(searchTextEdit->text(), QString("ZZZ"));
@@ -1072,6 +1111,7 @@ void TestGui::testDeleteEntry()
 {
     // Add canned entries for consistent testing
     addCannedEntries();
+    checkStatusBarText("4 Ent");
 
     auto* groupView = m_dbWidget->findChild<GroupView*>("groupView");
     auto* entryView = m_dbWidget->findChild<EntryView*>("entryView");
@@ -1101,6 +1141,8 @@ void TestGui::testDeleteEntry()
         QCOMPARE(m_db->metadata()->recycleBin()->entries().size(), 1);
     }
 
+    checkStatusBarText("3 Ent");
+
     // Select multiple entries and move them to the recycling bin
     clickIndex(entryView->model()->index(1, 1), entryView, Qt::LeftButton);
     clickIndex(entryView->model()->index(2, 1), entryView, Qt::LeftButton, Qt::ControlModifier);
@@ -1185,6 +1227,7 @@ void TestGui::testCloneEntry()
     Entry* entryClone = entryView->entryFromIndex(entryView->model()->index(1, 1));
     QVERIFY(entryOrg->uuid() != entryClone->uuid());
     QCOMPARE(entryClone->title(), entryOrg->title() + QString(" - Clone"));
+    QVERIFY(m_dbWidget->currentSelectedEntry()->uuid() == entryClone->uuid());
 }
 
 void TestGui::testEntryPlaceholders()
@@ -1868,6 +1911,14 @@ void TestGui::checkSaveDatabase()
     QFAIL("Could not save database.");
 }
 
+void TestGui::checkStatusBarText(const QString& textFragment)
+{
+    QApplication::processEvents();
+    QVERIFY(m_statusBarLabel->isVisible());
+    QTRY_VERIFY2(m_statusBarLabel->text().startsWith(textFragment),
+                 qPrintable(QString("'%1' doesn't start with '%2'").arg(m_statusBarLabel->text(), textFragment)));
+}
+
 void TestGui::triggerAction(const QString& name)
 {
     auto* action = m_mainWindow->findChild<QAction*>(name);
