@@ -18,15 +18,11 @@
 #include "MockChallengeResponseKey.h"
 
 MockChallengeResponseKey::MockChallengeResponseKey(const QByteArray& secret)
-    : ChallengeResponseKey(QUuid("aac5b480-cdc0-411e-9cb8-962062dcc1fd"))
+    : ChallengeResponseKey()
     , m_secret(secret)
 {
 }
 
-MockChallengeResponseKey::~MockChallengeResponseKey()
-{
-}
-
 QByteArray MockChallengeResponseKey::rawKey() const
 {
     return m_challenge + m_secret;
